@@ -626,7 +626,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   FILE *fp = NULL;
 
   DBG_INIT ();
-  DBG (2, "sane_init (authorize = %p)\n", (void *) authorize);
+  DBG (2, "sane_init (authorize %s null)\n", (authorize) ? "!=" : "==");
   if (version_code)
     *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 /*  fp = sanei_config_open (AS6E_CONFIG_FILE);*/
