@@ -30,18 +30,18 @@
 #include <unistd.h>
 #include <time.h>
 
-#include "_stdint.h"
+#include "../include/_stdint.h"
 
-#include <sane/sane.h>
-#include <sane/sanei.h>
-#include <sane/saneopts.h>
-#include <sane/sanei_scsi.h>
-#include <sane/sanei_usb.h>
-#include <sane/sanei_debug.h>
-#include <sane/sanei_config.h>
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
+#include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_debug.h"
+#include "../include/sane/sanei_config.h"
 
 #define BACKEND_NAME coolscan3
-#include <sane/sanei_backend.h>	/* must be last */
+#include "../include/sane/sanei_backend.h"	/* must be last */
 
 #define CS3_VERSION_MAJOR 1
 #define CS3_VERSION_MINOR 0
@@ -451,15 +451,18 @@ sane_open(SANE_String_Const name, SANE_Handle * h)
 			/*o.cap |= SANE_CAP_INACTIVE; */
 			break;
 
-/*		case CS3_OPTION_INFRARED:
+		case CS3_OPTION_INFRARED:
 			o.name = "infrared";
 			o.title = "Read infrared channel";
 			o.desc = "Read infrared channel in addition to scan colors";
 			o.type = SANE_TYPE_BOOL;
 			o.size = WSIZE;
 			o.cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
+#ifndef SANE_FRAME_RGBI
+                        o.cap |= SANE_CAP_INACTIVE;
+#endif
 			break;
-*/
+
 		case CS3_OPTION_DEPTH:
 			o.name = "depth";
 			o.title = "Bit depth per channel";
@@ -967,8 +970,7 @@ sane_open(SANE_String_Const name, SANE_Handle * h)
 			o.cap = SANE_CAP_SOFT_SELECT | SANE_CAP_SOFT_DETECT;
 			break;
 		default:
-			DBG(1, "BUG: sane_open(): Unknown option number.\n");
-			return SANE_STATUS_INVAL;
+			DBG(1, "BUG: sane_open(): Unknown option number: %d\n", i_option);
 			break;
 		}
 		s->option_list[i_option] = o;
@@ -1396,15 +1398,16 @@ sane_get_parameters(SANE_Handle h, SANE_Parameters * p)
 	p->bytes_per_line =
 		s->n_colors * s->logical_width * s->bytes_per_pixel;
 
-	if (s->infrared) {
 #ifdef SANE_FRAME_RGBI
+	if (s->infrared) {
 		p->format = SANE_FRAME_RGBI;
-#else
-		p->format = SANE_FRAME_RGB;
-#endif
+
 	} else {
+#endif
 		p->format = SANE_FRAME_RGB;	/* XXXXXXXX CCCCCCCCCC */
+#ifdef SANE_FRAME_RGBI
 	}
+#endif
 
 	p->last_frame = SANE_TRUE;
 	p->lines = s->logical_height;
