@@ -18,16 +18,15 @@ enum fujitsu_Option
   OPT_STANDARD_GROUP,
   OPT_SOURCE, /*fb/adf/front/back/duplex*/
   OPT_MODE,   /*mono/gray/color*/
-  OPT_X_RES,  /*a range or a list*/
-  OPT_Y_RES,  /*a range or a list*/
+  OPT_RES,    /*a range or a list*/
 
   OPT_GEOMETRY_GROUP,
+  OPT_PAGE_WIDTH,
+  OPT_PAGE_HEIGHT,
   OPT_TL_X,
   OPT_TL_Y,
   OPT_BR_X,
   OPT_BR_Y,
-  OPT_PAGE_WIDTH,
-  OPT_PAGE_HEIGHT,
 
   OPT_ENHANCEMENT_GROUP,
   OPT_BRIGHTNESS,
@@ -62,6 +61,7 @@ enum fujitsu_Option
   OPT_VARIANCE,
 
   OPT_ADVANCED_GROUP,
+  OPT_AWD,
   OPT_ALD,
   OPT_COMPRESS,
   OPT_COMPRESS_ARG,
@@ -70,6 +70,9 @@ enum fujitsu_Option
   OPT_DF_THICKNESS,
   OPT_DF_LENGTH,
   OPT_DF_DIFF,
+  OPT_DF_RECOVERY,
+  OPT_PAPER_PROTECT,
+  OPT_STAPLE_DETECT,
   OPT_BG_COLOR,
   OPT_DROPOUT_COLOR,
   OPT_BUFF_MODE,
@@ -81,6 +84,7 @@ enum fujitsu_Option
   OPT_BLUE_OFFSET,
   OPT_LOW_MEM,
   OPT_SIDE,
+  OPT_HWDESKEWCROP,
   OPT_SWDESKEW,
   OPT_SWDESPECK,
   OPT_SWCROP,
@@ -153,29 +157,14 @@ struct fujitsu
   /* members in order found in scsi data...                                */
   int basic_x_res;
   int basic_y_res;
-  int step_x_res;
-  int step_y_res;
+  int step_x_res[6]; /*one for each mode*/
+  int step_y_res[6]; /*one for each mode*/
   int max_x_res;
   int max_y_res;
   int min_x_res;
   int min_y_res;
 
-  int std_res_200;
-  int std_res_180;
-  int std_res_160;
-  int std_res_150;
-  int std_res_120;
-  int std_res_100;
-  int std_res_75;
-  int std_res_60;
-  int std_res_1200;
-  int std_res_800;
-  int std_res_600;
-  int std_res_480;
-  int std_res_400;
-  int std_res_320;
-  int std_res_300;
-  int std_res_240;
+  int std_res[16]; /*some scanners only support a few resolutions*/
 
   /* max scan size in pixels comes from scanner in basic res units */
   int max_x_basic;
@@ -237,7 +226,10 @@ struct fujitsu
   int has_cmd_subwindow;
   int has_cmd_endorser;
   int has_cmd_hw_status;
+  int has_cmd_hw_status_2;
+  int has_cmd_hw_status_3;
   int has_cmd_scanner_ctl;
+  int has_cmd_device_restart;
 
   /*FIXME: do we need the vendor window param list? */
 
@@ -250,6 +242,10 @@ struct fujitsu
   int num_internal_dither;
   int num_download_dither;
 
+  int has_df_recovery;
+  int has_paper_protect;
+  int has_staple_detect;
+
   int has_rif;
   int has_dtc;
   int has_sdtc;
@@ -262,6 +258,7 @@ struct fujitsu
   int has_diffusion;
   int has_ipc3;
   int has_rotation;
+  int has_hybrid_crop_deskew;
 
   int has_comp_MH;
   int has_comp_MR;
@@ -284,7 +281,7 @@ struct fujitsu
   /* --------------------------------------------------------------------- */
   /* immutable values which are gathered by mode_sense command     */
 
-  int has_MS_unknown;
+  int has_MS_autocolor;
   int has_MS_prepick;
   int has_MS_sleep;
   int has_MS_duplex;
@@ -348,10 +345,8 @@ struct fujitsu
   SANE_String_Const mode_list[7];
   SANE_String_Const source_list[5];
 
-  SANE_Int x_res_list[17];
-  SANE_Int y_res_list[17];
-  SANE_Range x_res_range;
-  SANE_Range y_res_range;
+  SANE_Int res_list[17];
+  SANE_Range res_range;
 
   /*geometry group*/
   SANE_Range tl_x_range;
@@ -382,6 +377,9 @@ struct fujitsu
   SANE_Range compress_arg_range;
   SANE_String_Const df_action_list[4];
   SANE_String_Const df_diff_list[5];
+  SANE_String_Const df_recovery_list[4];
+  SANE_String_Const paper_protect_list[4];
+  SANE_String_Const staple_detect_list[4];
   SANE_String_Const bg_color_list[4];
   SANE_String_Const do_color_list[5];
   SANE_String_Const lamp_color_list[5];
@@ -454,6 +452,7 @@ struct fujitsu
   int variance;
 
   /*advanced group*/
+  int awd;
   int ald;
   int compress;
   int compress_arg;
@@ -462,6 +461,9 @@ struct fujitsu
   int df_thickness;
   int df_length;
   int df_diff;
+  int df_recovery;
+  int paper_protect;
+  int staple_detect;
   int bg_color;
   int dropout_color;
   int buff_mode;
@@ -473,6 +475,7 @@ struct fujitsu
   int green_offset;
   int blue_offset;
   int low_mem;
+  int hwdeskewcrop;
   int swdeskew;
   int swdespeck;
   int swcrop;
@@ -533,6 +536,11 @@ struct fujitsu
 
   unsigned char * buffers[2];
 
+  /* --------------------------------------------------------------------- */
+  /*hardware feature bookkeeping*/
+  int req_driv_crop;
+  int req_driv_lut;
+
   /* --------------------------------------------------------------------- */
   /* values used by the software enhancment code (deskew, crop, etc)       */
   SANE_Status deskew_stat;
@@ -677,14 +685,6 @@ struct fujitsu
 #  define PATH_MAX 1024
 #endif
 
-#ifndef PATH_SEP
-#ifdef HAVE_OS2_H
-#  define PATH_SEP       '\\'
-#else
-#  define PATH_SEP       '/'
-#endif
-#endif
-
 /* ------------------------------------------------------------------------- */
 
 SANE_Status sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize);
@@ -781,6 +781,7 @@ static SANE_Status mode_select_auto (struct fujitsu *s);
 
 static SANE_Status set_sleep_mode(struct fujitsu *s);
 
+static int must_fully_buffer (struct fujitsu *s);
 static int get_page_width (struct fujitsu *s);
 static int get_page_height (struct fujitsu *s);
 
@@ -788,7 +789,7 @@ static SANE_Status send_lut (struct fujitsu *s);
 static SANE_Status send_endorser (struct fujitsu *s);
 static SANE_Status endorser (struct fujitsu *s);
 static SANE_Status set_window (struct fujitsu *s);
-static SANE_Status get_pixelsize(struct fujitsu *s);
+static SANE_Status get_pixelsize(struct fujitsu *s, int actual);
 
 static SANE_Status update_params (struct fujitsu *s);
 static SANE_Status backup_params (struct fujitsu *s);
