@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2003, 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
-   Copyright (C) 2005-2010 Stephane Voltz <stef.dev@free.fr>
+   Copyright (C) 2005-2012 Stephane Voltz <stef.dev@free.fr>
    Copyright (C) 2006 Laurent Charpentier <laurent_pubs@yahoo.com>
    Copyright (C) 2009 Pierre Willenbrock <pierre@pirsoft.dnsalias.org>
    
@@ -49,6 +49,8 @@
 
 #include "genesys_low.h"
 
+#define FREE_IFNOT_NULL(x)		if(x!=NULL) { free(x); x=NULL;}
+
 #define ENABLE(OPTION)  s->opt[OPTION].cap &= ~SANE_CAP_INACTIVE
 #define DISABLE(OPTION) s->opt[OPTION].cap |=  SANE_CAP_INACTIVE
 #define IS_ACTIVE(OPTION) (((s->opt[OPTION].cap) & SANE_CAP_INACTIVE) == 0)
@@ -91,9 +93,16 @@ enum Genesys_Option
   OPT_GAMMA_VECTOR_R,
   OPT_GAMMA_VECTOR_G,
   OPT_GAMMA_VECTOR_B,
+  OPT_SWDESKEW,
+  OPT_SWCROP,
+  OPT_SWDESPECK,
+  OPT_DESPECK,
+  OPT_SWSKIP,
+  OPT_SWDEROTATE,
 
   OPT_EXTRAS_GROUP,
   OPT_LAMP_OFF_TIME,
+  OPT_LAMP_OFF,
   OPT_THRESHOLD,
   OPT_THRESHOLD_CURVE,
   OPT_DISABLE_DYNAMIC_LINEART,
