@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2010-2011 Stphane Voltz <stef.dev@free.fr>
+   Copyright (C) 2010-2012 Stphane Voltz <stef.dev@free.fr>
    
     
    This file is part of the SANE package.
@@ -41,28 +41,9 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice. 
 */
-#include "../include/sane/config.h"
-
-#include <errno.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <math.h>
-
-#include "../include/sane/sane.h"
-#include "../include/sane/sanei.h"
-#include "../include/sane/saneopts.h"
-
-#ifndef HACK
 #undef BACKEND_NAME
 #define BACKEND_NAME genesys_low
-#endif
 
-#include "../include/sane/sanei_backend.h"
-#include "../include/sane/sanei_config.h"
-#include "../include/sane/sanei_usb.h"
-
-#include "../include/_stdint.h"
 #include "genesys_low.h"
 
 /* ------------------------------------------------------------------------ */
@@ -250,11 +231,11 @@ SANE_Status
 sanei_genesys_read_hregister (Genesys_Device * dev, uint8_t reg, uint8_t * val)
 {
   SANE_Status status;
-  uint16_t value;
+  SANE_Byte value[2];
 
   status =
     sanei_usb_control_msg (dev->dn, REQUEST_TYPE_IN, REQUEST_BUFFER,
-			   0x100 | VALUE_GET_REGISTER, 0x22+(reg<<8), 2, (SANE_Byte *)&value);
+			   0x100 | VALUE_GET_REGISTER, 0x22+(reg<<8), 2, value);
   if (status != SANE_STATUS_GOOD)
     {
       DBG (DBG_error,
@@ -262,11 +243,11 @@ sanei_genesys_read_hregister (Genesys_Device * dev, uint8_t reg, uint8_t * val)
 	   reg, sane_strstatus (status));
       return status;
     }
-  *val=value & 0xff;
-  DBG( DBG_io2, "sanei_genesys_read_hregister(0x%02x)=0x%02x\n",reg,value & 0xff);
+  *val=value[0];
+  DBG( DBG_io2, "sanei_genesys_read_hregister(0x%02x)=0x%02x\n",reg,*val);
 
   /* check usb link status */
-  if((value & 0xff00) != 0x5500)
+  if((value[1] & 0xff) != 0x55)
     {
       DBG (DBG_error,"sanei_genesys_read_hregister: invalid read, scanner unplugged ?\n");
       status=SANE_STATUS_IO_ERROR;
@@ -372,11 +353,11 @@ static SANE_Status
 sanei_genesys_read_gl847_register (Genesys_Device * dev, uint8_t reg, uint8_t * val)
 {
   SANE_Status status;
-  uint16_t value;
+  SANE_Byte value[2];
 
   status =
     sanei_usb_control_msg (dev->dn, REQUEST_TYPE_IN, REQUEST_BUFFER,
-			   VALUE_GET_REGISTER, 0x22+(reg<<8), 2, (SANE_Byte *)&value);
+			   VALUE_GET_REGISTER, 0x22+(reg<<8), 2, value);
   if (status != SANE_STATUS_GOOD)
     {
       DBG (DBG_error,
@@ -384,11 +365,11 @@ sanei_genesys_read_gl847_register (Genesys_Device * dev, uint8_t reg, uint8_t *
 	   reg, sane_strstatus (status));
       return status;
     }
-  *val=value & 0xff;
-  DBG( DBG_io2, "sanei_genesys_read_gl847_register(0x%02x)=0x%02x\n",reg,value & 0xff);
+  *val=value[0];
+  DBG( DBG_io2, "sanei_genesys_read_gl847_register(0x%02x)=0x%02x\n",reg,*val);
 
   /* check usb link status */
-  if((value & 0xff00) != 0x5500)
+  if((value[1] & 0xff) != 0x55)
     {
       DBG (DBG_error,"sanei_genesys_read_gl847_register: invalid read, scanner unplugged ?\n");
       status=SANE_STATUS_IO_ERROR;
@@ -940,7 +921,7 @@ sanei_genesys_read_data_from_scanner (Genesys_Device * dev, uint8_t * data,
 	  time_count++;
 	}
     }
-  while ((time_count < 2500) && (words == 0));
+  while ((time_count < 2500*2) && (words == 0));
 
   if (words == 0)		/* timeout, buffer does not get filled */
     {
@@ -1009,7 +990,7 @@ SANE_Status
 sanei_genesys_write_ahb (SANE_Int dn, uint32_t addr, uint32_t size, uint8_t * data)
 {
   uint8_t outdata[8];
-  size_t written;
+  size_t written,blksize;
   SANE_Status status = SANE_STATUS_GOOD;
   int i;
   char msg[60]="AHB=";
@@ -1045,14 +1026,28 @@ sanei_genesys_write_ahb (SANE_Int dn, uint32_t addr, uint32_t size, uint8_t * da
     }
 
   /* write actual data */
-  written = size;
-  status = sanei_usb_write_bulk (dn, data, &written);
-  if (status != SANE_STATUS_GOOD)
+  written = 0;
+  do
     {
-      DBG (DBG_error, "sanei_genesys_write_ahb: failed while writing bulk data: %s\n",
-	   sane_strstatus (status));
-      return status;
+      if (size - written > BULKOUT_MAXSIZE)
+        {
+          blksize = BULKOUT_MAXSIZE;
+        }
+      else
+        {
+          blksize = size - written;
+        }
+      status = sanei_usb_write_bulk (dn, data + written, &blksize);
+      if (status != SANE_STATUS_GOOD)
+        {
+          DBG (DBG_error,
+               "sanei_genesys_write_ahb: failed while writing bulk data: %s\n",
+               sane_strstatus (status));
+          return status;
+        }
+      written += blksize;
     }
+  while (written < size);
 
   return status;
 }
@@ -1069,7 +1064,7 @@ sanei_genesys_wait_for_home (Genesys_Device * dev)
 
   DBGSTART;
 
-  /* clearthe parking status whatever the outcome of the function */
+  /* clear the parking status whatever the outcome of the function */
   dev->parking=SANE_FALSE;
 
   /* read initial status, if head isn't at home and motor is on
@@ -1108,6 +1103,10 @@ sanei_genesys_wait_for_home (Genesys_Device * dev)
 	       sane_strstatus (status));
 	  return status;
 	}
+          if (DBG_LEVEL >= DBG_io2)
+            {
+              sanei_genesys_print_status (val);
+            }
       ++loop;
     }
   while (loop < 300 && !(val & HOMESNR) && status == SANE_STATUS_GOOD);
@@ -1116,4 +1115,366 @@ sanei_genesys_wait_for_home (Genesys_Device * dev)
   return status;
 }
 
+/**@brief compute hardware sensor dpi to use
+ * compute the sensor hardware dpi based on target resolution.
+ * A lower dpihw enable faster scans.
+ * @param dev device used for the scan
+ * @param xres x resolution of the scan
+ * @return the hardware dpi to use
+ */
+int sanei_genesys_compute_dpihw(Genesys_Device *dev, int xres)
+{
+  /* can't be below 600 dpi */
+  if (xres <= 600)
+    {
+      return 600;
+    }
+  if (xres <= dev->sensor.optical_res / 4)
+    {
+      return dev->sensor.optical_res / 4;
+    }
+  if (xres <= dev->sensor.optical_res / 2)
+    {
+      return dev->sensor.optical_res / 2;
+    }
+  return dev->sensor.optical_res;
+}
+
+/** @get motor profile
+ * search for the database of motor profiles and get the best one. Each
+ * profile is at full step and at a reference exposure. Use first entry
+ * by default.
+ * @param motors motor profile database
+ * @param motor_type motor id
+ * @param exposure exposure time
+ * @return a pointer to a Motor_Profile struct
+ */
+Motor_Profile *sanei_genesys_get_motor_profile(Motor_Profile *motors, int motor_type, int exposure)
+{
+  unsigned int i;
+  int idx;
+
+  i=0;
+  idx=-1;
+  while(motors[i].exposure!=0)
+    {
+      /* exact match */
+      if(motors[i].motor_type==motor_type && motors[i].exposure==exposure)
+        {
+          return &(motors[i]);
+        }
+
+      /* closest match */
+      if(motors[i].motor_type==motor_type)
+        {
+          /* if profile exposure is higher than the required one,
+           * the entry is a candidate for the closest match */
+          if(motors[i].exposure>=exposure)
+            {
+              if(idx<0)
+                {
+                  /* no match found yet */
+                  idx=i;
+                }
+              else
+                {
+                  /* test for better match */
+                  if(motors[i].exposure<motors[idx].exposure)
+                    {
+                      idx=i;
+                    }
+                }
+            }
+        }
+      i++;
+    }
+
+  /* default fallback */
+  if(idx<0)
+    {
+      DBG (DBG_warn,"%s: using default motor profile\n",__FUNCTION__);
+      idx=0;
+    }
+
+  return &(motors[idx]);
+}
+
+/**@brief compute motor step type to use
+ * compute the step type (full, half, quarter, ...) to use based
+ * on target resolution
+ * @param motors motor profile database
+ * @param motor_type motor id
+ * @param exposure sensor exposure
+ * @return 0 for full step
+ *         1 for half step
+ *         2 for quarter step
+ *         3 for eighth step
+ */
+int sanei_genesys_compute_step_type(Motor_Profile *motors,
+                                    int motor_type,
+                                    int exposure)
+{
+Motor_Profile *profile;
+
+    profile=sanei_genesys_get_motor_profile(motors, motor_type, exposure);
+    return profile->step_type;
+}
+
+/** @brief generate slope table
+ * Generate the slope table to use for the scan using a reference slope
+ * table.
+ * @param slope pointer to the slope table to fill
+ * @param steps pointer to return used step number
+ * @param dpi   desired motor resolution
+ * @param exposure exposure used
+ * @param base_dpi base resolution of the motor
+ * @param step_type step type used for scan
+ * @param factor shrink factor for the slope
+ * @param motor_type motor id
+ * @param motors motor profile database
+ */
+int sanei_genesys_slope_table(uint16_t *slope,
+		             int       *steps,
+			     int       dpi,
+			     int       exposure,
+			     int       base_dpi,
+			     int       step_type,
+			     int       factor,
+                             int       motor_type,
+                             Motor_Profile *motors)
+{
+int sum, i;
+uint16_t target,current;
+Motor_Profile *profile;
+
+	/* required speed */
+	target=((exposure * dpi) / base_dpi)>>step_type;
+        DBG (DBG_io2, "%s: target=%d\n", __FUNCTION__, target);
+	
+	/* fill result with target speed */
+        for(i=0;i<SLOPE_TABLE_SIZE;i++)
+          slope[i]=target;
+
+        profile=sanei_genesys_get_motor_profile(motors, motor_type, exposure);
+
+	/* use profile to build table */
+        i=0;
+	sum=0;
+
+        /* first step is used unmodified */
+        current=profile->table[0];
+
+        /* loop on profile copying and apply step type */
+        while(profile->table[i]!=0 && current>=target)
+          {
+            slope[i]=current;
+            sum+=slope[i];
+            i++;
+            current=profile->table[i]>>step_type;
+          }
+
+        /* range checking */
+        if(profile->table[i]==0 && DBG_LEVEL >= DBG_warn && current>target)
+          {
+            DBG (DBG_warn,"%s: short slope table, failed to reach %d. target too low ?\n",__FUNCTION__,target);
+          }
+        if(i<3 && DBG_LEVEL >= DBG_warn)
+          {
+            DBG (DBG_warn,"%s: short slope table, failed to reach %d. target too high ?\n",__FUNCTION__,target);
+          }
+
+        /* align on factor */
+        while(i%factor!=0)
+          {
+            slope[i+1]=slope[i];
+            sum+=slope[i];
+            i++;
+          }
+
+        /* ensure minimal slope size */
+        while(i<2*factor)
+          {
+            slope[i+1]=slope[i];
+            sum+=slope[i];
+            i++;
+          }
+
+        /* return used steps and acceleration sum */
+        *steps=i/factor;
+	return sum;
+}
+
+/** @brief returns the lowest possible ydpi for the device
+ * Parses device entry to find lowest motor dpi.
+ * @param dev device description
+ * @return lowest motor resolution
+ */
+int sanei_genesys_get_lowest_ydpi(Genesys_Device *dev)
+{
+  int min=20000;
+  int i=0;
+
+  while(dev->model->ydpi_values[i]!=0)
+    {
+      if(dev->model->ydpi_values[i]<min)
+        {
+          min=dev->model->ydpi_values[i];
+        }
+      i++;
+    }
+  return min;
+}
+
+/** @brief returns the lowest possible dpi for the device
+ * Parses device entry to find lowest motor or sensor dpi.
+ * @param dev device description
+ * @return lowest motor resolution
+ */
+int sanei_genesys_get_lowest_dpi(Genesys_Device *dev)
+{
+  int min=20000;
+  int i=0;
+
+  while(dev->model->ydpi_values[i]!=0)
+    {
+      if(dev->model->ydpi_values[i]<min)
+        {
+          min=dev->model->ydpi_values[i];
+        }
+      i++;
+    }
+  i=0;
+  while(dev->model->xdpi_values[i]!=0)
+    {
+      if(dev->model->xdpi_values[i]<min)
+        {
+          min=dev->model->xdpi_values[i];
+        }
+      i++;
+    }
+  return min;
+}
+
+/** @brief check is a cache entry may be used
+ * Compares current settings with the cache entry and return
+ * SANE_TRUE if they are compatible.
+ */
+SANE_Status
+sanei_genesys_is_compatible_calibration (Genesys_Device * dev,
+				 Genesys_Calibration_Cache * cache,
+				 int for_overwrite)
+{
+#ifdef HAVE_SYS_TIME_H
+  struct timeval time;
+#endif
+  int compatible = 1, resolution;
+  SANE_Status status;
+
+  DBGSTART;
+ 
+  if(dev->model->cmd_set->calculate_current_setup==NULL)
+    {
+      DBG (DBG_proc,
+	   "sanei_genesys_is_compatible_calibration: no calculate_setup, non compatible cache\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+
+  status = dev->model->cmd_set->calculate_current_setup (dev);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_error,
+	   "sanei_genesys_is_compatible_calibration: failed to calculate current setup: %s\n",
+	   sane_strstatus (status));
+      return status;
+    }
+  dev->current_setup.scan_method = dev->settings.scan_method;
+
+  DBG (DBG_proc, "sanei_genesys_is_compatible_calibration: checking\n");
+  
+  /* a calibration cache is compatible if color mode and x dpi match the user 
+   * requested scan. In the case of CIS scanners, dpi isn't a criteria */
+  if (dev->model->is_cis == SANE_FALSE)
+    {
+      resolution = dev->settings.xres;
+      if(resolution>dev->sensor.optical_res)
+        {
+          resolution=dev->sensor.optical_res;
+        }
+      compatible = (resolution == ((int) cache->used_setup.xres));
+    }
+  else
+    {
+      resolution=sanei_genesys_compute_dpihw(dev,dev->settings.xres);
+      compatible = (resolution == ((int) sanei_genesys_compute_dpihw(dev,cache->used_setup.xres)));
+    }
+  if (dev->current_setup.scan_method != cache->used_setup.scan_method)
+    {
+      DBG (DBG_io,
+	   "sanei_genesys_is_compatible_calibration: current method=%d, used=%d\n",
+	   dev->current_setup.scan_method, cache->used_setup.scan_method);
+      compatible = 0;
+    }
+  if (!compatible)
+    {
+      DBG (DBG_proc,
+	   "sanei_genesys_is_compatible_calibration: completed, non compatible cache\n");
+      return SANE_STATUS_UNSUPPORTED;
+    }
+
+  /* a cache entry expires after 60 minutes for non sheetfed scanners */
+  /* this is not taken into account when overwriting cache entries    */
+#ifdef HAVE_SYS_TIME_H
+  if(for_overwrite == SANE_FALSE)
+    {
+      gettimeofday (&time, NULL);
+      if ((time.tv_sec - cache->last_calibration > 60 * 60)
+          && (dev->model->is_sheetfed == SANE_FALSE)
+          && (dev->settings.scan_method == SCAN_METHOD_FLATBED))
+        {
+          DBG (DBG_proc,
+               "sanei_genesys_is_compatible_calibration: expired entry, non compatible cache\n");
+          return SANE_STATUS_UNSUPPORTED;
+        }
+    }
+#endif
+
+  DBGCOMPLETED;
+  return SANE_STATUS_GOOD;
+}
+
+
+/** @brief compute maximum line distance shift
+ * compute maximum line distance shift for the motor and sensor
+ * combination. Line distance shift is the distance between different
+ * color component of CCD sensors. Since these components aren't at
+ * the same physical place, they scan diffrent lines. Software must
+ * take this into account to accurately mix color data.
+ * @param dev device session to compute max_shift for
+ * @param channels number of color channels for the scan
+ * @param yres motor resolution used for the scan
+ * @param flags scan flags
+ * @return 0 or line distance shift
+ */
+int sanei_genesys_compute_max_shift(Genesys_Device *dev,
+                                    int channels,
+                                    int yres,
+                                    int flags)
+{
+  int max_shift;
+
+  max_shift=0;
+  if (channels > 1 && !(flags & SCAN_FLAG_IGNORE_LINE_DISTANCE))
+    {
+      max_shift = dev->ld_shift_r;
+      if (dev->ld_shift_b > max_shift)
+	max_shift = dev->ld_shift_b;
+      if (dev->ld_shift_g > max_shift)
+	max_shift = dev->ld_shift_g;
+      max_shift = (max_shift * yres) / dev->motor.base_ydpi;
+    }
+  return max_shift;
+}
+
+
+
 /* vim: set sw=2 cino=>2se-1sn-1s{s^-1st0(0u0 smarttab expandtab: */
