@@ -2,7 +2,9 @@
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
    AFE offset/gain setting by David Stevenson <david.stevenson@zoom.co.uk>
-   Copyright (C) 2002 - 2006 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2007 Henning Geinitz <sane@geinitz.org>
+   Copyright (C) 2009 Stphane Voltz <stef.dev@free.fr> for sheetfed
+                      calibration code.
 
    This file is part of the SANE package.
    
@@ -324,6 +326,7 @@ SANE_Status
 gt68xx_scanner_new (GT68xx_Device * dev, GT68xx_Scanner ** scanner_return)
 {
   GT68xx_Scanner *scanner;
+  int i;
 
   *scanner_return = NULL;
 
@@ -341,6 +344,15 @@ gt68xx_scanner_new (GT68xx_Device * dev, GT68xx_Scanner ** scanner_return)
   scanner->cal_g = NULL;
   scanner->cal_b = NULL;
 
+  for(i=0;i<MAX_RESOLUTIONS;i++)
+    {
+      scanner->calibrations[i].dpi = 0;
+      scanner->calibrations[i].red = NULL;
+      scanner->calibrations[i].green = NULL;
+      scanner->calibrations[i].blue = NULL;
+      scanner->calibrations[i].gray = NULL;
+    }
+
   *scanner_return = scanner;
   return SANE_STATUS_GOOD;
 }
@@ -376,6 +388,8 @@ gt68xx_scanner_free_calibrators (GT68xx_Scanner * scanner)
 SANE_Status
 gt68xx_scanner_free (GT68xx_Scanner * scanner)
 {
+  int i;
+
   if (!scanner)
     {
       DBG (5, "gt68xx_scanner_free: scanner==NULL\n");
@@ -390,6 +404,28 @@ gt68xx_scanner_free (GT68xx_Scanner * scanner)
 
   gt68xx_scanner_free_calibrators (scanner);
 
+  /* free in memory calibration data */
+  for (i = 0; i < MAX_RESOLUTIONS; i++)
+    {
+      scanner->calibrations[i].dpi = 0;
+      if (scanner->calibrations[i].red != NULL)
+	{
+	  gt68xx_calibrator_free (scanner->calibrations[i].red);
+	}
+      if (scanner->calibrations[i].green != NULL)
+	{
+	  gt68xx_calibrator_free (scanner->calibrations[i].green);
+	}
+      if (scanner->calibrations[i].blue != NULL)
+	{
+	  gt68xx_calibrator_free (scanner->calibrations[i].blue);
+	}
+      if (scanner->calibrations[i].gray != NULL)
+	{
+	  gt68xx_calibrator_free (scanner->calibrations[i].gray);
+	}
+    }
+
   free (scanner);
 
   return SANE_STATUS_GOOD;
@@ -525,7 +561,7 @@ gt68xx_scanner_start_scan_extended (GT68xx_Scanner * scanner,
     }
 
   if (scanner->dev->model->is_cis 
-      && !(scanner->dev->model->flags & GT68XX_FLAG_SHEET_FED))
+      && !((scanner->dev->model->flags & GT68XX_FLAG_SHEET_FED) && scanner->calibrated == SANE_FALSE))
     {
       status =
 	gt68xx_device_set_exposure_time (scanner->dev,
@@ -1883,15 +1919,15 @@ gt68xx_afe_cis_auto (GT68xx_Scanner * scanner)
 				      r_gbuffer, g_gbuffer, b_gbuffer));
       if (!red_done)
 	red_done =
-	  gt68xx_afe_cis_adjust_exposure ("red", &values, r_gbuffer, 248,
+	  gt68xx_afe_cis_adjust_exposure ("red", &values, r_gbuffer, 245,
 					  &exposure->r_time);
       if (!green_done)
 	green_done =
-	  gt68xx_afe_cis_adjust_exposure ("green", &values, g_gbuffer, 248,
+	  gt68xx_afe_cis_adjust_exposure ("green", &values, g_gbuffer, 245,
 					  &exposure->g_time);
       if (!blue_done)
 	blue_done =
-	  gt68xx_afe_cis_adjust_exposure ("blue", &values, b_gbuffer, 248,
+	  gt68xx_afe_cis_adjust_exposure ("blue", &values, b_gbuffer, 245,
 					  &exposure->b_time);
       exposure_count++;
       total_count++;
@@ -1901,6 +1937,15 @@ gt68xx_afe_cis_auto (GT68xx_Scanner * scanner)
   if (!red_done || !green_done || !blue_done)
     DBG (0, "gt68xx_afe_cis_auto: setting exposure reached limit\n");
 
+  /* store afe calibration when needed */
+  if(scanner->dev->model->flags & GT68XX_FLAG_HAS_CALIBRATE)
+    {
+      memcpy(&(scanner->afe_params), afe, sizeof(GT68xx_AFE_Parameters));
+      scanner->exposure_params.r_time=exposure->r_time;
+      scanner->exposure_params.g_time=exposure->g_time;
+      scanner->exposure_params.b_time=exposure->b_time;
+    }
+
   free (r_gbuffer);
   free (g_gbuffer);
   free (b_gbuffer);
@@ -1912,5 +1957,788 @@ gt68xx_afe_cis_auto (GT68xx_Scanner * scanner)
   return SANE_STATUS_GOOD;
 }
 
+/** @brief create and copy calibrator
+ * Creates a calibrator of the given width and copy data from reference
+ * to initialize it
+ * @param calibator pointer to the calibrator to create
+ * @param reference calibrator with reference data to copy
+ * @param width the width in pixels of the calibrator
+ * @param offset offset in pixels when copying data from reference
+ * @return SANE_STATUS_GOOD and a filled calibrator if enough memory
+ */
+static SANE_Status
+gt68xx_calibrator_create_copy (GT68xx_Calibrator ** calibrator,
+			       GT68xx_Calibrator * reference, int width,
+			       int offset)
+{
+  SANE_Status status;
+  int i;
+
+  if (reference == NULL)
+    {
+      DBG (1, "gt68xx_calibrator_create_copy: NULL reference, skipping...\n");
+      *calibrator = NULL;
+      return SANE_STATUS_GOOD;
+    }
+  /* check for reference overflow */
+  if(width+offset>reference->width)
+    {
+      DBG (1, "gt68xx_calibrator_create_copy: required with and offset exceed reference width\n");
+      return SANE_STATUS_INVAL;
+    }
+
+  status = gt68xx_calibrator_new (width, 65535, calibrator);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (1,
+	   "gt68xx_calibrator_create_copy: failed to create calibrator: %s\n",
+	   sane_strstatus (status));
+      return status;
+    }
+
+  for(i=0;i<width;i++)
+    {
+      (*calibrator)->k_white[i]=reference->k_white[i+offset];
+      (*calibrator)->k_black[i]=reference->k_black[i+offset];
+      (*calibrator)->white_line[i]=reference->white_line[i+offset];
+      (*calibrator)->black_line[i]=reference->black_line[i+offset];
+    }
+
+  return status;
+}
+ 
+static SANE_Status
+gt68xx_sheetfed_move_to_scan_area (GT68xx_Scanner * scanner,
+				  GT68xx_Scan_Request * request)
+{
+  SANE_Status status;
+
+  if (!(scanner->dev->model->flags & GT68XX_FLAG_SHEET_FED)
+      || scanner->dev->model->command_set->move_paper == NULL)
+    return SANE_STATUS_GOOD;
+
+  /* send move paper command */
+  RIE (scanner->dev->model->command_set->move_paper (scanner->dev, request));
+
+  /* wait until paper is set to the desired position */
+  return gt68xx_scanner_wait_for_positioning (scanner);
+}
+
+/**< number of consecutive white line to detect a white area */
+#define WHITE_LINES 2
+
+/** @brief calibrate sheet fed scanner
+ * This function calibrates sheet fed scanner by scanning a calibration
+ * target (which may be a blank page). It first move to a white area then
+ * does afe and exposure calibration. Then it scans white lines to get data
+ * for shading correction.
+ * @param scanner structure describing the frontend session and the device 
+ * @return SANE_STATUS_GOOD is everything goes right, SANE_STATUS_INVAL 
+ * otherwise.
+ */
+static SANE_Status
+gt68xx_sheetfed_scanner_calibrate (GT68xx_Scanner * scanner)
+{
+  SANE_Status status;
+  GT68xx_Scan_Request request;
+  GT68xx_Scan_Parameters params;
+  int count, i, x, y, white;
+  unsigned int *buffer_pointers[3];
+#ifdef DEBUG_CALIBRATION
+  FILE *fcal;
+  char title[50];
+#endif
+
+  DBG (3, "gt68xx_sheetfed_scanner_calibrate: start.\n");
+
+  /* clear calibration if needed */
+  gt68xx_scanner_free_calibrators (scanner);
+  for (i = 0; i < MAX_RESOLUTIONS; i++)
+    {
+      if(scanner->calibrations[i].red!=NULL)
+        {
+          gt68xx_calibrator_free (scanner->calibrations[i].red);
+        }
+      if(scanner->calibrations[i].green!=NULL)
+        {
+          gt68xx_calibrator_free (scanner->calibrations[i].green);
+        }
+      if(scanner->calibrations[i].blue!=NULL)
+        {
+          gt68xx_calibrator_free (scanner->calibrations[i].blue);
+        }
+      if(scanner->calibrations[i].gray!=NULL)
+        {
+          gt68xx_calibrator_free (scanner->calibrations[i].gray);
+        }
+    }
+  scanner->calibrated = SANE_FALSE;
+
+  /* find minimum horizontal resolution */
+  request.xdpi = 9600;
+  for (i = 0; scanner->dev->model->xdpi_values[i] != 0; i++)
+    {
+      if (scanner->dev->model->xdpi_values[i] < request.xdpi)
+	{
+	  request.xdpi = scanner->dev->model->xdpi_values[i];
+	  request.ydpi = scanner->dev->model->xdpi_values[i];
+	}
+    }
+
+  /* move to white area SA_CALIBRATE uses its own y0/ys fixed values */
+  request.x0 = 0;
+  request.y0 = scanner->dev->model->y_offset_calib;
+  request.xs = scanner->dev->model->x_size;
+  request.depth = 8;
+
+  request.color = SANE_FALSE;
+  request.mbs = SANE_TRUE;
+  request.mds = SANE_TRUE;
+  request.mas = SANE_FALSE;
+  request.lamp = SANE_TRUE;
+  request.calculate = SANE_FALSE;
+  request.use_ta = SANE_FALSE;
+  request.backtrack = SANE_FALSE;
+  request.backtrack_lines = 0;
+
+  /* skip start of calibration sheet */
+  status = gt68xx_sheetfed_move_to_scan_area (scanner, &request);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (1,
+	   "gt68xx_sheetfed_scanner_calibrate: failed to skip start of calibration sheet %s\n",
+	   sane_strstatus (status));
+      return status;
+    }
+
+  status = gt68xx_device_lamp_control (scanner->dev, SANE_FALSE, SANE_TRUE);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (1,
+	   "gt68xx_sheetfed_scanner_calibrate: gt68xx_device_lamp_control returned %s\n",
+	   sane_strstatus (status));
+      return status;
+    }
+
+  /* loop until we find a white area to calibrate on */
+  i = 0;
+  request.y0 = 0;
+  do
+    {
+      /* start scan */
+      status =
+	gt68xx_scanner_start_scan_extended (scanner, &request, SA_CALIBRATE,
+					    &params);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_sheetfed_scanner_calibrate: gt68xx_scanner_start_scan_extended returned %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+
+      /* loop until we find WHITE_LINES consecutive white lines or we reach and of area */
+      white = 0;
+      y = 0;
+      do
+	{
+	  status = gt68xx_line_reader_read (scanner->reader, buffer_pointers);
+	  if (status != SANE_STATUS_GOOD)
+	    {
+	      DBG (1,
+		   "gt68xx_sheetfed_scanner_calibrate: gt68xx_line_reader_read returned %s\n",
+		   sane_strstatus (status));
+	      gt68xx_scanner_stop_scan (scanner);
+	      return status;
+	    }
+
+	  /* check for white line */
+	  count = 0;
+	  for (x = 0; x < params.pixel_xs; x++)
+	    {
+	      if (((buffer_pointers[0][x] >> 8) & 0xff) > 50)
+		{
+		  count++;
+		}
+	    }
+
+	  /* line is white if 93% is above black level */
+	  if ((100 * count) / params.pixel_xs < 93)
+	    {
+	      white = 0;
+	    }
+	  else
+	    {
+	      white++;
+	    }
+	  y++;
+	}
+      while ((white < WHITE_LINES) && (y < params.pixel_ys));
+
+      /* end scan */
+      gt68xx_scanner_stop_scan (scanner);
+
+      i++;
+    }
+  while (i < 20 && white < WHITE_LINES);
+
+  /* check if we found a white area */
+  if (white != WHITE_LINES)
+    {
+      DBG (1,
+	   "gt68xx_sheetfed_scanner_calibrate: didn't find a white area\n");
+      return SANE_STATUS_INVAL;
+    }
+
+  /* now do calibration */
+  scanner->auto_afe = SANE_TRUE;
+  scanner->calib = SANE_TRUE;
+
+  /* loop at each possible xdpi to create calibrators */
+  i = 0;
+  while (scanner->dev->model->xdpi_values[i] > 0)
+    {
+      request.xdpi = scanner->dev->model->xdpi_values[i];
+      request.ydpi = scanner->dev->model->xdpi_values[i];
+      request.x0 = 0;
+      request.y0 = 0;
+      request.xs = scanner->dev->model->x_size;
+      request.color = SANE_FALSE;
+      request.mbs = SANE_FALSE;
+      request.mds = SANE_TRUE;
+      request.mas = SANE_FALSE;
+      request.lamp = SANE_TRUE;
+      request.calculate = SANE_FALSE;
+      request.use_ta = SANE_FALSE;
+      request.backtrack = SANE_FALSE;
+      request.backtrack_lines = 0;
+
+      /* calibrate in color */
+      request.color = SANE_TRUE;
+      status = gt68xx_scanner_calibrate (scanner, &request);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_sheetfed_scanner_calibrate: gt68xx_scanner_calibrate returned %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+
+      /* since auto afe is done at a fixed resolution, we don't need to
+       * do each each time, once is enough */
+      scanner->auto_afe = SANE_FALSE;
+
+      /* allocate and save per dpi calibrators */
+      scanner->calibrations[i].dpi = request.xdpi;
+
+      /* recompute params */
+      request.calculate = SANE_TRUE;
+      gt68xx_device_setup_scan (scanner->dev, &request, SA_SCAN, &params);
+
+      scanner->calibrations[i].pixel_x0 = params.pixel_x0;
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->calibrations[i].red),
+				       scanner->cal_r, scanner->cal_r->width,
+				       0);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_sheetfed_scanner_calibrate: failed to create red calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->calibrations[i].green),
+				       scanner->cal_g, scanner->cal_g->width,
+				       0);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_sheetfed_scanner_calibrate: failed to create green calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->calibrations[i].blue),
+				       scanner->cal_b, scanner->cal_b->width,
+				       0);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_sheetfed_scanner_calibrate: failed to create blue calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+
+      /* calibrate in gray */
+      request.color = SANE_FALSE;
+      status = gt68xx_scanner_calibrate (scanner, &request);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_sheetfed_scanner_calibrate: gt68xx_scanner_calibrate returned %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+
+      if (scanner->cal_gray)
+	{
+	  status =
+	    gt68xx_calibrator_create_copy (&(scanner->calibrations[i].gray),
+					   scanner->cal_gray,
+					   scanner->cal_gray->width, 0);
+	  if (status != SANE_STATUS_GOOD)
+	    {
+	      DBG (1,
+		   "gt68xx_sheetfed_scanner_calibrate: failed to create gray calibrator: %s\n",
+		   sane_strstatus (status));
+	      return status;
+	    }
+	}
+
+#ifdef DEBUG_CALIBRATION
+      sprintf (title, "cal-%03d-red.pnm", scanner->calibrations[i].dpi);
+      fcal = fopen (title, "wb");
+      if (fcal != NULL)
+	{
+	  fprintf (fcal, "P5\n%d 1\n255\n", params.pixel_xs);
+	  for (x = 0; x < params.pixel_xs; x++)
+	    fputc ((scanner->calibrations[i].red->k_white[x] >> 8) & 0xff,
+		   fcal);
+	  fclose (fcal);
+	}
+      sprintf (title, "cal-%03d-green.pnm", scanner->calibrations[i].dpi);
+      fcal = fopen (title, "wb");
+      if (fcal != NULL)
+	{
+	  fprintf (fcal, "P5\n%d 1\n255\n", params.pixel_xs);
+	  for (x = 0; x < params.pixel_xs; x++)
+	    fputc ((scanner->calibrations[i].green->k_white[x] >> 8) & 0xff,
+		   fcal);
+	  fclose (fcal);
+	}
+      sprintf (title, "cal-%03d-blue.pnm", scanner->calibrations[i].dpi);
+      fcal = fopen (title, "wb");
+      if (fcal != NULL)
+	{
+	  fprintf (fcal, "P5\n%d 1\n255\n", params.pixel_xs);
+	  for (x = 0; x < params.pixel_xs; x++)
+	    fputc ((scanner->calibrations[i].blue->k_white[x] >> 8) & 0xff,
+		   fcal);
+	  fclose (fcal);
+	}
+#endif
+
+      /* next resolution */
+      i++;
+    }
+
+  scanner->calibrated = SANE_TRUE;
+
+  /* eject calibration target from feeder */
+  gt68xx_device_paperfeed (scanner->dev);
+
+  /* save calibration to file */
+  gt68xx_write_calibration (scanner);
+
+  DBG (3, "gt68xx_sheetfed_scanner_calibrate: end.\n");
+  return SANE_STATUS_GOOD;
+}
+
+/** @brief assign calibration for scan
+ * This function creates the calibrators and set up afe for the requested
+ * scan. It uses calibration data that has been created by 
+ * gt68xx_sheetfed_scanner_calibrate.
+ * @param scanner structure describing the frontend session and the device 
+ * @return SANE_STATUS_GOOD is everything goes right, SANE_STATUS_INVAL 
+ * otherwise.
+ */
+static SANE_Status
+gt68xx_assign_calibration (GT68xx_Scanner * scanner,
+			   GT68xx_Scan_Parameters params)
+{
+  int i, dpi, offset;
+  SANE_Status status = SANE_STATUS_GOOD;
+
+  DBG (3, "gt68xx_assign_calibration: start.\n");
+
+  dpi = params.xdpi;
+  DBG (4, "gt68xx_assign_calibration: searching calibration for %d dpi\n",
+       dpi);
+
+  /* search matching dpi */
+  i = 0;
+  while (scanner->calibrations[i].dpi > 0
+	 && scanner->calibrations[i].dpi != dpi)
+    {
+      i++;
+    }
+
+  /* check if found a match */
+  if (scanner->calibrations[i].dpi == 0)
+    {
+      DBG (4,
+	   "gt68xx_assign_calibration: failed to find calibration for %d dpi\n",
+	   dpi);
+      return SANE_STATUS_INVAL;
+    }
+  DBG (4, "gt68xx_assign_calibration: using entry %d for %d dpi\n", i, dpi);
+
+  DBG (5,
+       "gt68xx_assign_calibration: using scan_parameters: pixel_x0=%d, pixel_xs=%d \n",
+       params.pixel_x0, params.pixel_xs);
+
+  /* AFE/exposure data copy */
+  memcpy (scanner->dev->afe, &(scanner->afe_params),
+	  sizeof (GT68xx_AFE_Parameters));
+  scanner->dev->exposure->r_time = scanner->exposure_params.r_time;
+  scanner->dev->exposure->g_time = scanner->exposure_params.g_time;
+  scanner->dev->exposure->b_time = scanner->exposure_params.b_time;
+
+  /* free calibrators if needed */
+  gt68xx_scanner_free_calibrators (scanner);
+
+  /* TODO compute offset based on the x0 value from scan_request */
+  offset = params.pixel_x0 - scanner->calibrations[i].pixel_x0;
+
+  /* calibrator allocation and copy */
+  if (scanner->calibrations[i].red!=NULL)
+    {
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->cal_r),
+				       scanner->calibrations[i].red,
+				       params.pixel_xs,
+                                       offset);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_assign_calibration: failed to create calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+    }
+
+  if (scanner->calibrations[i].green!=NULL)
+    {
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->cal_g),
+				       scanner->calibrations[i].green,
+				       params.pixel_xs,
+				       offset);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_assign_calibration: failed to create calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+    }
+
+  if (scanner->calibrations[i].blue!=NULL)
+    {
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->cal_b),
+				       scanner->calibrations[i].blue,
+				       params.pixel_xs,
+				       offset);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_assign_calibration: failed to create calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+    }
+
+  if (scanner->calibrations[i].gray!=NULL)
+    {
+      status =
+	gt68xx_calibrator_create_copy (&(scanner->cal_gray),
+				       scanner->calibrations[i].gray,
+				       params.pixel_xs,
+				       offset);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (1,
+	       "gt68xx_assign_calibration: failed to create calibrator: %s\n",
+	       sane_strstatus (status));
+	  return status;
+	}
+    }
+
+  DBG (3, "gt68xx_assign_calibration: end.\n");
+  return status;
+}
+
+static char *gt68xx_calibration_file(GT68xx_Scanner * scanner)
+{
+  char *ptr=NULL;
+  char tmp_str[PATH_MAX];
+
+  ptr=getenv("HOME");
+  if(ptr!=NULL)
+    {
+      sprintf (tmp_str, "%s/.sane/gt68xx-%s.cal", ptr, scanner->dev->model->name);
+    }
+  else
+    {
+      ptr=getenv("TMPDIR");
+      if(ptr!=NULL)
+        {
+          sprintf (tmp_str, "%s/gt68xx-%s.cal", ptr, scanner->dev->model->name);
+        }
+      else
+        {
+          sprintf (tmp_str, "/tmp/gt68xx-%s.cal", scanner->dev->model->name);
+        }
+    }
+  DBG(5,"gt68xx_calibration_file: using >%s< for calibration file name\n",tmp_str);
+  return strdup(tmp_str);
+}
+
+static SANE_Status
+gt68xx_clear_calibration (GT68xx_Scanner * scanner)
+{
+  char *fname;
+  int i;
+
+  if (scanner->calibrated == SANE_FALSE)
+    return SANE_STATUS_GOOD;
+
+  /* clear file */
+  fname = gt68xx_calibration_file (scanner);
+  unlink (fname);
+  free (fname);
+
+  /* free calibrators */
+  for (i = 0; i < MAX_RESOLUTIONS && scanner->calibrations[i].dpi > 0; i++)
+    {
+      scanner->calibrations[i].dpi = 0;
+      if (scanner->calibrations[i].red)
+	gt68xx_calibrator_free (scanner->calibrations[i].red);
+      if (scanner->calibrations[i].green)
+	gt68xx_calibrator_free (scanner->calibrations[i].green);
+      if (scanner->calibrations[i].blue)
+	gt68xx_calibrator_free (scanner->calibrations[i].blue);
+      if (scanner->calibrations[i].gray)
+	gt68xx_calibrator_free (scanner->calibrations[i].gray);
+    }
+
+  /* reset flags */
+  scanner->calibrated = SANE_FALSE;
+  scanner->val[OPT_QUALITY_CAL].w = SANE_FALSE;
+  scanner->val[OPT_NEED_CALIBRATION_SW].w = SANE_TRUE;
+  DBG (5, "gt68xx_clear_calibration: done\n");
+  return SANE_STATUS_GOOD;
+}
+
+static SANE_Status
+gt68xx_write_calibration (GT68xx_Scanner * scanner)
+{
+  char *fname;
+  FILE *fcal;
+  int i;
+  SANE_Int nullwidth = 0;
+
+  if (scanner->calibrated == SANE_FALSE)
+    {
+      return SANE_STATUS_GOOD;
+    }
+
+  /* open file */
+  fname = gt68xx_calibration_file (scanner);
+  fcal = fopen (fname, "wb");
+  free (fname);
+  if (fcal == NULL)
+    {
+      DBG (1,
+	   "gt68xx_write_calibration: failed to open calibration file for writing %s\n",
+	   strerror (errno));
+      return SANE_STATUS_IO_ERROR;
+    }
+
+  /* TODO we save check endianness and word alignment in case of a home
+   * directory used trough different archs */
+  fwrite (&(scanner->afe_params), sizeof (GT68xx_AFE_Parameters), 1, fcal);
+  fwrite (&(scanner->exposure_params), sizeof (GT68xx_Exposure_Parameters), 1,
+	  fcal);
+  for (i = 0; i < MAX_RESOLUTIONS && scanner->calibrations[i].dpi > 0; i++)
+    {
+      DBG (1, "gt68xx_write_calibration: saving %d dpi calibration\n",
+	   scanner->calibrations[i].dpi);
+      fwrite (&(scanner->calibrations[i].dpi), sizeof (SANE_Int), 1, fcal);
+      fwrite (&(scanner->calibrations[i].pixel_x0), sizeof (SANE_Int), 1,
+	      fcal);
+
+      fwrite (&(scanner->calibrations[i].red->width), sizeof (SANE_Int), 1,
+	      fcal);
+      fwrite (&(scanner->calibrations[i].red->white_level), sizeof (SANE_Int),
+	      1, fcal);
+      fwrite (scanner->calibrations[i].red->k_white, sizeof (unsigned int),
+	      scanner->calibrations[i].red->width, fcal);
+      fwrite (scanner->calibrations[i].red->k_black, sizeof (unsigned int),
+	      scanner->calibrations[i].red->width, fcal);
+      fwrite (scanner->calibrations[i].red->white_line, sizeof (double),
+	      scanner->calibrations[i].red->width, fcal);
+      fwrite (scanner->calibrations[i].red->black_line, sizeof (double),
+	      scanner->calibrations[i].red->width, fcal);
+
+      fwrite (&(scanner->calibrations[i].green->width), sizeof (SANE_Int), 1,
+	      fcal);
+      fwrite (&(scanner->calibrations[i].green->white_level),
+	      sizeof (SANE_Int), 1, fcal);
+      fwrite (scanner->calibrations[i].green->k_white, sizeof (unsigned int),
+	      scanner->calibrations[i].green->width, fcal);
+      fwrite (scanner->calibrations[i].green->k_black, sizeof (unsigned int),
+	      scanner->calibrations[i].green->width, fcal);
+      fwrite (scanner->calibrations[i].green->white_line, sizeof (double),
+	      scanner->calibrations[i].green->width, fcal);
+      fwrite (scanner->calibrations[i].green->black_line, sizeof (double),
+	      scanner->calibrations[i].green->width, fcal);
+
+      fwrite (&(scanner->calibrations[i].blue->width), sizeof (SANE_Int), 1,
+	      fcal);
+      fwrite (&(scanner->calibrations[i].blue->white_level),
+	      sizeof (SANE_Int), 1, fcal);
+      fwrite (scanner->calibrations[i].blue->k_white, sizeof (unsigned int),
+	      scanner->calibrations[i].blue->width, fcal);
+      fwrite (scanner->calibrations[i].blue->k_black, sizeof (unsigned int),
+	      scanner->calibrations[i].blue->width, fcal);
+      fwrite (scanner->calibrations[i].blue->white_line, sizeof (double),
+	      scanner->calibrations[i].blue->width, fcal);
+      fwrite (scanner->calibrations[i].blue->black_line, sizeof (double),
+	      scanner->calibrations[i].blue->width, fcal);
+
+      if (scanner->calibrations[i].gray != NULL)
+	{
+	  fwrite (&(scanner->calibrations[i].gray->width), sizeof (SANE_Int),
+		  1, fcal);
+	  fwrite (&(scanner->calibrations[i].gray->white_level),
+		  sizeof (SANE_Int), 1, fcal);
+	  fwrite (scanner->calibrations[i].gray->k_white,
+		  sizeof (unsigned int), scanner->calibrations[i].gray->width,
+		  fcal);
+	  fwrite (scanner->calibrations[i].gray->k_black,
+		  sizeof (unsigned int), scanner->calibrations[i].gray->width,
+		  fcal);
+	  fwrite (scanner->calibrations[i].gray->white_line, sizeof (double),
+		  scanner->calibrations[i].gray->width, fcal);
+	  fwrite (scanner->calibrations[i].gray->black_line, sizeof (double),
+		  scanner->calibrations[i].gray->width, fcal);
+	}
+      else
+	{
+	  fwrite (&nullwidth, sizeof (SANE_Int), 1, fcal);
+	}
+    }
+  DBG (5, "gt68xx_write_calibration: wrote %d calibrations\n", i);
+
+  fclose (fcal);
+  return SANE_STATUS_GOOD;
+}
+
+static SANE_Status
+gt68xx_read_calibration (GT68xx_Scanner * scanner)
+{
+  char *fname;
+  FILE *fcal;
+  int i;
+  SANE_Int width, level;
+
+  scanner->calibrated = SANE_FALSE;
+  fname = gt68xx_calibration_file (scanner);
+  fcal = fopen (fname, "rb");
+  free (fname);
+  if (fcal == NULL)
+    {
+      DBG (1,
+	   "gt68xx_read_calibration: failed to open calibration file for reading %s\n",
+	   strerror (errno));
+      return SANE_STATUS_IO_ERROR;
+    }
+
+  /* TODO we should check endiannes and word alignment in case of a home
+   * directory used trough different archs */
+
+  /* TODO check for errors */
+  fread (&(scanner->afe_params), sizeof (GT68xx_AFE_Parameters), 1, fcal);
+  fread (&(scanner->exposure_params), sizeof (GT68xx_Exposure_Parameters), 1,
+	 fcal);
+
+  /* loop on calibrators */
+  i = 0;
+  fread (&(scanner->calibrations[i].dpi), sizeof (SANE_Int), 1, fcal);
+  while (!feof (fcal) && scanner->calibrations[i].dpi > 0)
+    {
+      fread (&(scanner->calibrations[i].pixel_x0), sizeof (SANE_Int), 1,
+	     fcal);
+
+      fread (&width, sizeof (SANE_Int), 1, fcal);
+      fread (&level, sizeof (SANE_Int), 1, fcal);
+      gt68xx_calibrator_new (width, level, &(scanner->calibrations[i].red));
+      fread (scanner->calibrations[i].red->k_white, sizeof (unsigned int),
+	     width, fcal);
+      fread (scanner->calibrations[i].red->k_black, sizeof (unsigned int),
+	     width, fcal);
+      fread (scanner->calibrations[i].red->white_line, sizeof (double), width,
+	     fcal);
+      fread (scanner->calibrations[i].red->black_line, sizeof (double), width,
+	     fcal);
+
+      fread (&width, sizeof (SANE_Int), 1, fcal);
+      fread (&level, sizeof (SANE_Int), 1, fcal);
+      gt68xx_calibrator_new (width, level, &(scanner->calibrations[i].green));
+      fread (scanner->calibrations[i].green->k_white, sizeof (unsigned int),
+	     width, fcal);
+      fread (scanner->calibrations[i].green->k_black, sizeof (unsigned int),
+	     width, fcal);
+      fread (scanner->calibrations[i].green->white_line, sizeof (double),
+	     width, fcal);
+      fread (scanner->calibrations[i].green->black_line, sizeof (double),
+	     width, fcal);
+
+      fread (&width, sizeof (SANE_Int), 1, fcal);
+      fread (&level, sizeof (SANE_Int), 1, fcal);
+      gt68xx_calibrator_new (width, level, &(scanner->calibrations[i].blue));
+      fread (scanner->calibrations[i].blue->k_white, sizeof (unsigned int),
+	     width, fcal);
+      fread (scanner->calibrations[i].blue->k_black, sizeof (unsigned int),
+	     width, fcal);
+      fread (scanner->calibrations[i].blue->white_line, sizeof (double),
+	     width, fcal);
+      fread (scanner->calibrations[i].blue->black_line, sizeof (double),
+	     width, fcal);
+
+      fread (&width, sizeof (SANE_Int), 1, fcal);
+      if (width > 0)
+	{
+	  fread (&level, sizeof (SANE_Int), 1, fcal);
+	  gt68xx_calibrator_new (width, level,
+				 &(scanner->calibrations[i].gray));
+	  fread (scanner->calibrations[i].gray->k_white,
+		 sizeof (unsigned int), width, fcal);
+	  fread (scanner->calibrations[i].gray->k_black,
+		 sizeof (unsigned int), width, fcal);
+	  fread (scanner->calibrations[i].gray->white_line, sizeof (double),
+		 width, fcal);
+	  fread (scanner->calibrations[i].gray->black_line, sizeof (double),
+		 width, fcal);
+	}
+      /* prepare for nex resolution */
+      i++;
+      fread (&(scanner->calibrations[i].dpi), sizeof (SANE_Int), 1, fcal);
+    }
+
+  DBG (5, "gt68xx_read_calibration: read %d calibrations\n", i);
+  fclose (fcal);
+
+  scanner->val[OPT_QUALITY_CAL].w = SANE_TRUE;
+  scanner->val[OPT_NEED_CALIBRATION_SW].w = SANE_FALSE;
+  scanner->calibrated = SANE_TRUE;
+  return SANE_STATUS_GOOD;
+}
+
 
 /* vim: set sw=2 cino=>2se-1sn-1s{s^-1st0(0u0 smarttab expandtab: */
