@@ -76,7 +76,6 @@
 #include "ibm.h"
 #include "ibm-scsi.c"
 
-#define MM_PER_INCH	25.4
 #define MAX(a,b)	((a) > (b) ? (a) : (b))
 
 static int num_devices = 0;
@@ -156,14 +155,14 @@ attach (const char *devnam, Ibm_Device ** devp)
     }
 
   if (!(
-	(strncmp (ibuf.vendor, "IBM", 3) ==0
-         && strncmp (ibuf.product, "2456", 4) == 0)
-        || (strncmp (ibuf.vendor, "RICOH", 5) == 0 
-	    && strncmp (ibuf.product, "IS420", 5) == 0) 
-        || (strncmp (ibuf.vendor, "RICOH", 5) == 0 
-	    && strncmp (ibuf.product, "IS410", 5) == 0) 
-        || (strncmp (ibuf.vendor, "RICOH", 5) == 0 
-	    && strncmp (ibuf.product, "IS430", 5) == 0) 
+	(strncmp ((char *)ibuf.vendor, "IBM", 3) ==0
+         && strncmp ((char *)ibuf.product, "2456", 4) == 0)
+        || (strncmp ((char *)ibuf.vendor, "RICOH", 5) == 0 
+	    && strncmp ((char *)ibuf.product, "IS420", 5) == 0) 
+        || (strncmp ((char *)ibuf.vendor, "RICOH", 5) == 0 
+	    && strncmp ((char *)ibuf.product, "IS410", 5) == 0) 
+        || (strncmp ((char *)ibuf.vendor, "RICOH", 5) == 0 
+	    && strncmp ((char *)ibuf.product, "IS430", 5) == 0) 
 	))
     {
       DBG (1, "attach: device \"%s\" doesn't look like a scanner I know\n",
@@ -186,8 +185,8 @@ attach (const char *devnam, Ibm_Device ** devp)
    * Ignore this function ... seems to work ok
    * Suggested to George Murphy george@topfloor.ie by henning
    */
-  if (strncmp(ibuf.vendor, "RICOH", 5) != 0
-      && strncmp(ibuf.product, "IS420", 5) != 0)
+  if (strncmp((char *)ibuf.vendor, "RICOH", 5) != 0
+      && strncmp((char *)ibuf.product, "IS420", 5) != 0)
     {
       DBG (3, "attach: sending OBJECT POSITION\n");
       status = object_position (fd, OBJECT_POSITION_UNLOAD);
@@ -251,8 +250,8 @@ attach (const char *devnam, Ibm_Device ** devp)
   dev->sane.vendor = "IBM";
   str = malloc (16 + 1);
   memset (str, 0, sizeof (str));
-  strncpy (str, ibuf.product, sizeof(ibuf.product));
-  strncpy (str + sizeof(ibuf.revision), ibuf.revision, sizeof(ibuf.revision));
+  strncpy (str, (char *)ibuf.product, sizeof(ibuf.product));
+  strncpy (str + sizeof(ibuf.revision), (char *)ibuf.revision, sizeof(ibuf.revision));
   str[sizeof(ibuf.product) + sizeof(ibuf.revision)] = '\0';
   dev->sane.model = str;
   dev->sane.type = "flatbed scanner";
@@ -364,6 +363,7 @@ init_options (Ibm_Scanner * s)
 
   s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  s->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
   s->val[OPT_NUM_OPTS].w = NUM_OPTIONS;
 
@@ -539,15 +539,15 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   FILE *fp;
 
   DBG_INIT ();
-  DBG (11, ">> sane_init (authorize = %p)\n", authorize);
+  DBG (11, ">> sane_init (authorize %s null)\n", (authorize) ? "!=" : "==");
 
 #if defined PACKAGE && defined VERSION
   DBG (2, "sane_init: ibm backend version %d.%d-%d ("
-       PACKAGE " " VERSION ")\n", V_MAJOR, V_MINOR, BUILD);
+       PACKAGE " " VERSION ")\n", SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 #endif
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open(IBM_CONFIG_FILE);
   if (fp)
@@ -899,8 +899,8 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
         }
 
       mode = s->val[OPT_MODE].s;
-      if ((strcmp (mode, "Lineart") == 0) ||
-	  (strcmp (mode, "Halftone")) == 0)
+      if ((strcmp (mode, SANE_VALUE_SCAN_MODE_LINEART) == 0) ||
+	  (strcmp (mode, SANE_VALUE_SCAN_MODE_HALFTONE)) == 0)
         {
           s->params.format = SANE_FRAME_GRAY;
           s->params.bytes_per_line = s->params.pixels_per_line / 8;
@@ -908,7 +908,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
           s->params.pixels_per_line = s->params.bytes_per_line * 8;
           s->params.depth = 1;
         }
-      else /* if (strcmp (mode, "Gray") == 0) */
+      else /* if (strcmp (mode, SANE_VALUE_SCAN_MODE_GRAY) == 0) */
         {
           s->params.format = SANE_FRAME_GRAY;
           s->params.bytes_per_line = s->params.pixels_per_line;
@@ -966,15 +966,15 @@ sane_start (SANE_Handle handle)
   s->brightness = s->val[OPT_BRIGHTNESS].w;
   s->contrast = s->val[OPT_CONTRAST].w;
   s->bpp = s->params.depth;
-  if (strcmp (mode_str, "Lineart") == 0) 
+  if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_LINEART) == 0) 
     {
       s->image_composition = IBM_BINARY_MONOCHROME;
     }
-  else if (strcmp (mode_str, "Halftone") == 0)
+  else if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_HALFTONE) == 0)
     {
       s->image_composition = IBM_DITHERED_MONOCHROME;
     }
-  else if (strcmp (mode_str, "Gray") == 0)
+  else if (strcmp (mode_str, SANE_VALUE_SCAN_MODE_GRAY) == 0)
     {
       s->image_composition = IBM_GRAYSCALE;
     }
@@ -1176,7 +1176,7 @@ SANE_Status
 sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
 {
   DBG (5, ">> sane_get_select_fd (handle = %p, fd = %p)\n",
-       handle, fd);
+       handle, (void *) fd);
   DBG (5, "<< sane_get_select_fd\n");
 
   return SANE_STATUS_UNSUPPORTED;
