@@ -15,7 +15,7 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-  $Id: niash.c,v 1.14 2008-11-26 21:21:28 kitno-guest Exp $
+  $Id$
 */
 
 /*
@@ -197,12 +197,12 @@ _UnityGammaTable (unsigned char *hwGamma)
 }
 
 static const SANE_Range rangeXmm = { 0, 220, 1 };
-static const SANE_Range rangeYmm = { 0, 297, 1 };
+static const SANE_Range rangeYmm = { 0, 296, 1 };
 static const SANE_Int startUpGamma = SANE_FIX (1.6);
 
-static const char colorStr[] = { "Color" };
-static const char grayStr[] = { "Gray" };
-static const char lineartStr[] = { "Lineart" };
+static const char colorStr[] = { SANE_VALUE_SCAN_MODE_COLOR };
+static const char grayStr[] = { SANE_VALUE_SCAN_MODE_GRAY };
+static const char lineartStr[] = { SANE_VALUE_SCAN_MODE_LINEART };
 
 #define DEPTH_LINEART  1
 #define DEPTH_GRAY     8
@@ -986,7 +986,6 @@ sane_control_option (SANE_Handle h, SANE_Int n, SANE_Action Action,
   TScanner *s;
   SANE_Bool fVal;
   static char szTable[100];
-  char szTemp[16];
   int *pi;
   int i;
   SANE_Int info;
@@ -1111,21 +1110,18 @@ sane_control_option (SANE_Handle h, SANE_Int n, SANE_Action Action,
             {
               if ((SANE_GAMMA_SIZE / 16) && (i % (SANE_GAMMA_SIZE / 16)) == 0)
                 {
-                  strcat (szTable, "\n");
-                  DBG (DBG_MSG, szTable);
-                  strcpy (szTable, "");
+                  DBG (DBG_MSG, "%s\n", szTable);
+		  szTable[0] = '\0';
                 }
               /* test for number print */
               if ((SANE_GAMMA_SIZE / 64) && (i % (SANE_GAMMA_SIZE / 64)) == 0)
                 {
-                  sprintf (szTemp, " %04X", pi[i]);
-                  strcat (szTable, szTemp);
+                  sprintf (szTable + strlen(szTable), " %04X", pi[i]);
                 }
             }
           if (strlen (szTable))
             {
-              strcat (szTable, "\n");
-              DBG (DBG_MSG, szTable);
+              DBG (DBG_MSG, "%s\n", szTable);
             }
           break;
 
