@@ -1,7 +1,8 @@
-/** SANE - Scanner Access Now Easy.
+/* SANE - Scanner Access Now Easy.
 
    Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
+   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -66,12 +67,14 @@
 extern const pixma_config_t pixma_mp150_devices[];
 extern const pixma_config_t pixma_mp750_devices[];
 extern const pixma_config_t pixma_mp730_devices[];
+extern const pixma_config_t pixma_mp810_devices[];
 extern const pixma_config_t pixma_iclass_devices[];
 
 static const pixma_config_t *const pixma_devices[] = {
   pixma_mp150_devices,
   pixma_mp750_devices,
   pixma_mp730_devices,
+  pixma_mp810_devices,
   pixma_iclass_devices,
   NULL
 };
@@ -302,6 +305,234 @@ pixma_get_time (time_t * sec, uint32_t * usec)
     *usec = tv.tv_usec;
 }
 
+/* convert 24/48 bit RGB to 8/16 bit grayscale
+ *
+ * Formular: g = (R + G + B) / 3
+ *
+ * sptr: source color scale buffer
+ * gptr: destination gray scale buffer
+ * c == 3: 24 bit RGB -> 8 bit gray
+ * c == 6: 48 bit RGB -> 16 bit gray
+ */
+uint8_t *
+pixma_rgb_to_gray (uint8_t * gptr, uint8_t * sptr, unsigned w, unsigned c)
+{
+  unsigned i, j, g;
+
+  /* PDBG (pixma_dbg (4, "*pixma_rgb_to_gray*****\n")); */
+
+  for (i = 0; i < w; i++)
+    {
+      for (j = 0, g = 0; j < 3; j++)
+        {
+          g += *sptr++;
+          if (c == 6) g += (*sptr++ << 8);      /* 48 bit RGB: high byte */
+        }
+
+      g /= 3;                                   /* 8 or 16 bit gray */
+      *gptr++ = g;
+      if (c == 6) *gptr++ = (g >> 8);           /* 16 bit gray: high byte */
+    }
+  return gptr;
+}
+
+/**
+ * This code was taken from the genesys backend
+ * uses threshold and threshold_curve to control software binarization
+ * @param sp    device set up for the scan
+ * @param dst   pointer where to store result
+ * @param src   pointer to raw data
+ * @param width width of the processed line
+ * @param c     1 for 1-channel single-byte data,
+ *              3 for 3-channel single-byte data,
+ *              6 for double-byte data
+ * */
+uint8_t *
+pixma_binarize_line(pixma_scan_param_t * sp, uint8_t * dst, uint8_t * src, unsigned width, unsigned c)
+{
+  unsigned j, x, windowX, sum = 0;
+  unsigned threshold;
+  unsigned offset, addCol;
+  int dropCol, offsetX;
+  unsigned char mask;
+  uint8_t min, max;
+
+  /* PDBG (pixma_dbg (4, "*pixma_binarize_line***** src = %u, dst = %u, width = %u, c = %u, threshold = %u, thershold_curve = %u *****\n",
+                      src, dst, width, c, sp->threshold, sp->threshold_curve)); */
+
+  /* 16 bit grayscale not supported */
+  if (c == 6)
+    {
+      PDBG (pixma_dbg (1, "*pixma_binarize_line***** Error: 16 bit grayscale not supported\n"));
+      return dst;
+    }
+
+  /* first, color convert to grayscale */
+    if (c != 1)
+      pixma_rgb_to_gray(dst, src, width, c);
+
+  /* second, normalize line */
+    min = 255;
+    max = 0;
+    for (x = 0; x < width; x++)
+      {
+        if (src[x] > max)
+          {
+            max = src[x];
+          }
+        if (src[x] < min)
+          {
+            min = src[x];
+          }
+      }
+
+    /* safeguard against dark or white areas */
+    if(min>80)
+        min=0;
+    if(max<80)
+        max=255;
+    for (x = 0; x < width; x++)
+      {
+        src[x] = ((src[x] - min) * 255) / (max - min);
+      }
+
+  /* third, create sliding window, prefill the sliding sum */
+    /* ~1mm works best, but the window needs to have odd # of pixels */
+    windowX = (6 * sp->xdpi) / 150;
+    if (!(windowX % 2))
+      windowX++;
+
+    /* to avoid conflicts with *dst start with offset */
+    offsetX = 1 + (windowX / 2) / 8;
+    for (j = offsetX; j <= windowX; j++)
+      sum += src[j];
+    /* PDBG (pixma_dbg (4, " *pixma_binarize_line***** windowX = %u, startX = %u, sum = %u\n",
+                     windowX, startX, sum)); */
+
+  /* fourth, walk the input buffer, output bits */
+    for (j = 0; j < width; j++)
+      {
+        /* output image location */
+        offset = j % 8;
+        mask = 0x80 >> offset;
+        threshold = sp->threshold;
+
+        /* move sum/update threshold only if there is a curve */
+        if (sp->threshold_curve)
+          {
+            addCol = j + windowX / 2;
+            dropCol = addCol - windowX;
+
+            if (dropCol >= offsetX && addCol < width)
+              {
+                sum += src[addCol];
+                sum -= (sum < src[dropCol] ? sum : src[dropCol]);       /* no negative sum */
+              }
+            threshold = sp->lineart_lut[sum / windowX];
+            /* PDBG (pixma_dbg (4, " *pixma_binarize_line***** addCol = %u, dropCol = %d, sum = %u, windowX = %u, lut-element = %d, threshold = %u\n",
+                             addCol, dropCol, sum, windowX, sum/windowX, threshold)); */
+          }
+
+        /* lookup threshold */
+        if (src[j] > threshold)
+            *dst &= ~mask;      /* white */
+        else
+            *dst |= mask;       /* black */
+
+        if (offset == 7)
+            dst++;
+      }
+
+  /* PDBG (pixma_dbg (4, " *pixma_binarize_line***** ready: src = %u, dst = %u *****\n", src, dst)); */
+
+  return dst;
+}
+
+/**
+   This code was taken from the genesys backend
+   Function to build a lookup table (LUT), often
+   used by scanners to implement brightness/contrast/gamma
+   or by backends to speed binarization/thresholding
+
+   offset and slope inputs are -127 to +127
+
+   slope rotates line around central input/output val,
+   0 makes horizontal line
+
+       pos           zero          neg
+       .       x     .             .  x
+       .      x      .             .   x
+   out .     x       .xxxxxxxxxxx  .    x
+       .    x        .             .     x
+       ....x.......  ............  .......x....
+            in            in            in
+
+   offset moves line vertically, and clamps to output range
+   0 keeps the line crossing the center of the table
+
+       high           low
+       .   xxxxxxxx   .
+       . x            .
+   out x              .          x
+       .              .        x
+       ............   xxxxxxxx....
+            in             in
+
+   out_min/max provide bounds on output values,
+   useful when building thresholding lut.
+   0 and 255 are good defaults otherwise.
+  * */
+static SANE_Status
+load_lut (unsigned char * lut,
+  int in_bits, int out_bits,
+  int out_min, int out_max,
+  int slope, int offset)
+{
+  int i, j;
+  double shift, rise;
+  int max_in_val = (1 << in_bits) - 1;
+  int max_out_val = (1 << out_bits) - 1;
+  unsigned char * lut_p = lut;
+
+  /* PDBG (pixma_dbg (4, "*load_lut***** start %d %d *****\n", slope, offset)); */
+
+  /* slope is converted to rise per unit run:
+   * first [-127,127] to [-1,1]
+   * then multiply by PI/2 to convert to radians
+   * then take the tangent (T.O.A)
+   * then multiply by the normal linear slope
+   * because the table may not be square, i.e. 1024x256*/
+  rise = tan((double)slope/127 * M_PI/2) * max_out_val / max_in_val;
+
+  /* line must stay vertically centered, so figure
+   * out vertical offset at central input value */
+  shift = (double)max_out_val/2 - (rise*max_in_val/2);
+
+  /* convert the user offset setting to scale of output
+   * first [-127,127] to [-1,1]
+   * then to [-max_out_val/2,max_out_val/2]*/
+  shift += (double)offset / 127 * max_out_val / 2;
+
+  for(i=0;i<=max_in_val;i++){
+    j = rise*i + shift;
+
+    if(j<out_min){
+      j=out_min;
+    }
+    else if(j>out_max){
+      j=out_max;
+    }
+
+    *lut_p=j;
+    lut_p++;
+  }
+
+  /* PDBG (pixma_dbg (4, "*load_lut***** finish *****\n")); */
+  /* PDBG (pixma_hexdump (4, lut, max_in_val+1)); */
+
+  return SANE_STATUS_GOOD;
+}
+
 int
 pixma_map_status_errno (unsigned status)
 {
@@ -378,7 +609,7 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
         {
           /* Write timeout is too low? */
           PDBG (pixma_dbg
-          (1, "ERROR:incomplete write, %u out of %u written\n",
+          (1, "ERROR: incomplete write, %u out of %u written\n",
            (unsigned) error, cmdlen));
           error = PIXMA_ETIMEDOUT;
         }
@@ -403,7 +634,7 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
   while (error == PIXMA_ETIMEDOUT && --tmo != 0);
   if (error < 0)
     {
-      PDBG (pixma_dbg (1, "WARNING:Error in response phase. cmd:%02x%02x\n",
+      PDBG (pixma_dbg (1, "WARNING: Error in response phase. cmd:%02x%02x\n",
 		       ((const uint8_t *) cmd)[0],
 		       ((const uint8_t *) cmd)[1]));
       PDBG (pixma_dbg (1,"  If the scanner hangs, reset it and/or unplug the "
@@ -544,7 +775,7 @@ pixma_close (pixma_t * s)
     {
       if (s->scanning)
 	{
-	  PDBG (pixma_dbg (3, "pixma_close():scanning in progress, call"
+	  PDBG (pixma_dbg (3, "pixma_close(): scanning in progress, call"
 			   " finish_scan()\n"));
 	  s->ops->finish_scan (s);
 	}
@@ -564,14 +795,21 @@ pixma_scan (pixma_t * s, pixma_scan_param_t * sp)
   if (error < 0)
     return error;
 
+  if (sp->mode == PIXMA_SCAN_MODE_LINEART)
+    {
+      load_lut(sp->lineart_lut, 8, 8, 50, 205,
+               sp->threshold_curve, sp->threshold-127);
+    }
+
 #ifndef NDEBUG
   pixma_dbg (3, "\n");
   pixma_dbg (3, "pixma_scan(): start\n");
-  pixma_dbg (3, "  line_size=%u image_size=%u channels=%u depth=%u\n",
+  pixma_dbg (3, "  line_size=%"PRIu64" image_size=%"PRIu64" channels=%u depth=%u\n",
 	     sp->line_size, sp->image_size, sp->channels, sp->depth);
   pixma_dbg (3, "  dpi=%ux%u offset=(%u,%u) dimension=%ux%u\n",
 	     sp->xdpi, sp->ydpi, sp->x, sp->y, sp->w, sp->h);
   pixma_dbg (3, "  gamma_table=%p source=%d\n", sp->gamma_table, sp->source);
+  pixma_dbg (3, "  threshold=%d threshold_curve=%d\n", sp->threshold, sp->threshold_curve);
 #endif
 
   s->param = sp;
@@ -601,7 +839,7 @@ fill_pixels (pixma_t * s, uint8_t * ptr, uint8_t * end, uint8_t value)
 {
   if (s->cur_image_size < s->param->image_size)
     {
-      int n = s->param->image_size - s->cur_image_size;
+      long n = s->param->image_size - s->cur_image_size;
       if (n > (end - ptr))
 	n = end - ptr;
       memset (ptr, value, n);
@@ -638,7 +876,7 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
       else
         {
           PDBG (pixma_dbg
-          (3, "pixma_read_image():completed (underrun detected)\n"));
+          (3, "pixma_read_image(): completed (underrun detected)\n"));
           s->scanning = 0;
         }
       return ib.wptr - (uint8_t *) buf;
@@ -659,7 +897,7 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
                 {
                   pixma_dbg (1, "WARNING:image size mismatches\n");
                   pixma_dbg (1,
-                       "    %u expected (%d lines) but %u received (%d lines)\n",
+                       "    %"PRIu64" expected (%d lines) but %"PRIu64" received (%"PRIu64" lines)\n",
                        s->param->image_size, s->param->h,
                        s->cur_image_size,
                        s->cur_image_size / s->param->line_size);
@@ -682,6 +920,7 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
               break;
             }
           s->cur_image_size += result;
+
           PASSERT (s->cur_image_size <= s->param->image_size);
         }
       if (ib.rptr)
@@ -700,7 +939,7 @@ cancel:
   s->scanning = 0;
   if (result == PIXMA_ECANCELED)
     {
-      PDBG (pixma_dbg (3, "pixma_read_image():cancelled by %sware\n",
+      PDBG (pixma_dbg (3, "pixma_read_image(): cancelled by %sware\n",
 		       (s->cancel) ? "soft" : "hard"));
     }
   else
@@ -757,18 +996,26 @@ pixma_wait_event (pixma_t * s, int timeout /*ms */ )
 int
 pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
 {
+  unsigned cfg_xdpi;
+
   if (!(sp->channels == 3 ||
 	(sp->channels == 1 && (s->cfg->cap & PIXMA_CAP_GRAY) != 0)))
     return PIXMA_EINVAL;
 
-  if (pixma_check_dpi (sp->xdpi, s->cfg->xdpi) < 0 ||
+  /* flatbed: use s->cfg->xdpi
+   * TPU/ADF: use s->cfg->ext_max_dpi, if configured with dpi value */
+  cfg_xdpi = ((sp->source == PIXMA_SOURCE_FLATBED
+               || s->cfg->ext_max_dpi == 0) ? s->cfg->xdpi
+                                            : s->cfg->ext_max_dpi);
+
+  if (pixma_check_dpi (sp->xdpi, cfg_xdpi) < 0 ||
       pixma_check_dpi (sp->ydpi, s->cfg->ydpi) < 0)
     return PIXMA_EINVAL;
 
   /* xdpi must be equal to ydpi except that
      xdpi = max_xdpi and ydpi = max_ydpi. */
   if (!(sp->xdpi == sp->ydpi ||
-	(sp->xdpi == s->cfg->xdpi && sp->ydpi == s->cfg->ydpi)))
+	(sp->xdpi == cfg_xdpi && sp->ydpi == s->cfg->ydpi)))
     return PIXMA_EINVAL;
 
   if (s->ops->check_param (s, sp) < 0)
@@ -821,7 +1068,7 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
 
   if (sp->depth == 0)
     sp->depth = 8;
-  if ((sp->depth % 8) != 0)
+  if ((sp->depth % 8) != 0 && sp->depth != 1)
     return PIXMA_EINVAL;
 
   sp->line_size = 0;
@@ -832,6 +1079,10 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
   if (sp->line_size == 0)
     sp->line_size = sp->depth / 8 * sp->channels * sp->w;
   sp->image_size = sp->line_size * sp->h;
+
+  /* image_size for software lineart is counted in bits */
+  if (sp->software_lineart == 1)
+    sp->image_size /= 8;
   return 0;
 }
 
