@@ -1,13 +1,9 @@
-/*.............................................................................
- * Project : linux driver for Plustek parallel-port scanners
- *.............................................................................
- * File:     plustek-pp_sysdep.h
- *           a trial to centralize changes between the different 
- *           kernel-versions some stuff is maybe not relevant, but anyway...
- *.............................................................................
+/* @file plustek-pp_sysdep.h
+ * @brief a trial to centralize changes between the different 
+ *        kernel-versions some stuff is maybe not relevant, but anyway...
  *
  * Copyright (C) 2000-2004 Gerhard Jaeger <gerhard@gjaeger.de>
- *.............................................................................
+ *
  * History:
  * 0.30 - initial version
  * 0.38 - added this header
@@ -17,6 +13,7 @@
  * 0.42 - added _GET_TIME
  *      - added LINUX_26 for new kernel
  *      - added _MINOR
+ * 0.43 - added class functions
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -62,7 +59,7 @@
 #define _SYSDEP_H_
 
 #ifndef LINUX_VERSION_CODE
-#  include <linux/version.h>
+# include <linux/version.h>
 #endif
 
 #ifndef VERSION_CODE
@@ -82,6 +79,7 @@
 #else
 #  define LINUX_24
 #  define LINUX_26
+# include <linux/device.h>
 #endif
 
 #include <linux/types.h> /* used later in this header */
@@ -305,6 +303,30 @@
 #  define _MINOR(p) minor(p->i_rdev)
 #endif
 
+/* Basic class macros */
+#ifdef LINUX_26
+#if LINUX_VERSION_CODE >= VERSION_CODE(2,6,15)
+
+typedef struct class class_t;
+#define CLASS_DEVICE_CREATE(cls, devt, device, fmt, arg...) class_device_create(cls, NULL, devt, device, fmt, ## arg)
+
+#else /* LINUX 2.6.0 - 2.6.14 */
+
+#if LINUX_VERSION_CODE >= VERSION_CODE(2,6,13) /* LINUX 2.6.13 - 2.6.14 */
+typedef struct class class_t;
+#define CLASS_DEVICE_CREATE class_device_create
+
+#else /* LINUX 2.6.0 - 2.6.12, class_simple */
+
+typedef struct class_simple class_t;
+#define CLASS_DEVICE_CREATE class_simple_device_add
+#define class_create class_simple_create
+#define class_destroy class_simple_destroy
+#define class_device_destroy(a, b) class_simple_device_remove(b)
+
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13) */
+#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15) */
+#endif
 
 #endif /* _SYSDEP_H_ */
 
