@@ -7,7 +7,7 @@
  *  @brief Calibration routines for CanoScan CIS devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de><br>
+ * Copyright (C) 2001-2007 Gerhard Jaeger <gerhard@gjaeger.de><br>
  * Large parts Copyright (C) 2003 Christopher Montgomery <monty@xiph.org>
  *
  * Montys' comment:
@@ -45,6 +45,10 @@
  *           lamp coarse calibration
  *         - added also speedtest
  *         - fixed segfault in fine calibration
+ * - 0.51  - added fine calibration cache
+ *         - usb_SwitchLamp() now really switches off the sensor
+ * - 0.52  - fixed setting for frontend values (gain/offset)
+ *         - added 0 pixel detection for offset calculation
  *
  * This file is part of the SANE package.
  *
@@ -91,26 +95,38 @@
  */
 #define _TWEAK_GAIN 1
 
+/* set the threshold for 0 pixels (in percent if pixels per line) */
+#define _DARK_TGT_THRESH 1
+
 /** 0 for not ready,  1 pos white lamp on,  2 lamp off */
 static int strip_state = 0;
 
 /** depending on the strip state, the sensor is moved to the shading position
  *  and the lamp ist switched on
  */
-static int cano_PrepareToReadWhiteCal( Plustek_Device *dev )
+static int
+cano_PrepareToReadWhiteCal( Plustek_Device *dev, SANE_Bool mv2shading_pos )
 {
-	HWDef *hw = &dev->usbDev.HwSetting;
+	SANE_Bool goto_shading_pos = SANE_TRUE;
+	HWDef     *hw = &dev->usbDev.HwSetting;
 	
 	switch (strip_state) {
 		case 0:
-			if(!usb_ModuleToHome( dev, SANE_TRUE )) {
-				DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
-				return _E_LAMP_NOT_IN_POS;
+			if( !usb_IsSheetFedDevice(dev)) {
+				if(!usb_ModuleToHome( dev, SANE_TRUE )) {
+					DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
+					return _E_LAMP_NOT_IN_POS;
+				}
+			} else {
+				goto_shading_pos = mv2shading_pos;
 			}
-			if( !usb_ModuleMove(dev, MOVE_Forward,
-				(u_long)dev->usbDev.pSource->ShadingOriginY)) {
-				DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
-				return _E_LAMP_NOT_IN_POS;
+
+			if( goto_shading_pos ) {
+				if( !usb_ModuleMove(dev, MOVE_Forward,
+					(u_long)dev->usbDev.pSource->ShadingOriginY)) {
+					DBG( _DBG_ERROR, "cano_PrepareToReadWhiteCal() failed\n" );
+					return _E_LAMP_NOT_IN_POS;
+				}
 			}
 	    	break;
 		case 2:
@@ -127,12 +143,14 @@ static int cano_PrepareToReadWhiteCal( Plustek_Device *dev )
 	return 0;
 }
 
-/**
+/** also here, depending on the strip state, the sensor will be moved to
+ * the shading position and the lamp will be switched off
  */
-static int cano_PrepareToReadBlackCal( Plustek_Device *dev )
+static int
+cano_PrepareToReadBlackCal( Plustek_Device *dev )
 {
 	if( strip_state == 0 )
-		if(cano_PrepareToReadWhiteCal(dev))
+		if(cano_PrepareToReadWhiteCal(dev, SANE_FALSE))
 			return SANE_FALSE;
 			
 	if( strip_state != 2 ) {
@@ -142,14 +160,14 @@ static int cano_PrepareToReadBlackCal( Plustek_Device *dev )
 		 */
 		if( dev->usbDev.pSource->DarkShadOrgY >= 0 ) {
 
-			usb_ModuleToHome( dev, SANE_TRUE );
+			if( !usb_IsSheetFedDevice(dev))
+				usb_ModuleToHome( dev, SANE_TRUE );
 			usb_ModuleMove  ( dev, MOVE_Forward,
 			                       (u_long)dev->usbDev.pSource->DarkShadOrgY );
 			dev->usbDev.a_bRegs[0x45] &= ~0x10;
 			strip_state = 0;
 
 		} else {
-
 		 	/* switch lamp off to read dark data... */
 			dev->usbDev.a_bRegs[0x29] = 0;
 			usb_switchLamp( dev, SANE_FALSE );
@@ -159,9 +177,10 @@ static int cano_PrepareToReadBlackCal( Plustek_Device *dev )
 	return 0;
 }
 
-/**
+/** according to the strip-state we switch the lamp on
  */
-static int cano_LampOnAfterCalibration( Plustek_Device *dev )
+static int
+cano_LampOnAfterCalibration( Plustek_Device *dev )
 {
 	HWDef *hw = &dev->usbDev.HwSetting;
 
@@ -231,17 +250,18 @@ cano_adjLampSetting( u_short *min, u_short *max, u_short *off, u_short val )
  * [Monty changes]: On the CanoScan at least, the default lamp
  * settings are several *hundred* percent too high and vary from
  * scanner-to-scanner by 20-50%. This is only for CIS devices 
- * where the lamp_off parameter is adjustable; I'd make it more general, 
+ * where the lamp_off parameter is adjustable; I'd make it more general,
  * but I only have the CIS hardware to test.
  */
-static int cano_AdjustLightsource( Plustek_Device *dev )
+static int
+cano_AdjustLightsource( Plustek_Device *dev )
 {
 	char         tmp[40];
 	int          i;
 	int          res_r, res_g, res_b;
 	u_long       dw, dwR, dwG, dwB, dwDiv, dwLoop1, dwLoop2;
 	RGBUShortDef max_rgb, min_rgb, tmp_rgb;
-	u_char      *scanbuf = dev->scanning.pScanBuffer;
+	u_long      *scanbuf = dev->scanning.pScanBuffer;
 	DCapsDef    *scaps   = &dev->usbDev.Caps;
 	HWDef       *hw      = &dev->usbDev.HwSetting;
 
@@ -250,17 +270,16 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 
 	DBG( _DBG_INFO, "cano_AdjustLightsource()\n" );
 
-	if( !(hw->bReg_0x26 & _ONE_CH_COLOR)) {
+	if( !usb_IsCISDevice(dev)) {
 		DBG( _DBG_INFO, "- function skipped, CCD device!\n" );
-		
-		if( !usb_Wait4Warmup( dev )) {
-			DBG( _DBG_ERROR, "cano_AdjustLightsource() - CANCEL detected\n" );
-			return SANE_FALSE;
-		}
-		return SANE_TRUE;	
+
+		/* HEINER: we might have to tweak the PWM for the lamps */
+		return SANE_TRUE;
 	}
 
-	/* define the strip to scan for coarse calibration;  done at 300dpi */
+	/* define the strip to scan for coarse calibration
+	 * done at optical resolution.
+	 */
 	m_ScanParam.Size.dwLines  = 1;
 	m_ScanParam.Size.dwPixels = scaps->Normal.Size.x *
 	                            scaps->OpticDpi.x / 300UL;
@@ -318,16 +337,16 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 
 		sprintf( tmp, "coarse-lamp-%u.raw", i );
 
-		dumpPicInit( &m_ScanParam, tmp );
-		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwPhyBytes );
+		dumpPicInit(&m_ScanParam, tmp);
+		dumpPic(tmp, (u_char*)scanbuf, m_ScanParam.Size.dwPhyBytes, 0);
 
 		if(usb_HostSwap())
 			usb_Swap((u_short *)scanbuf, m_ScanParam.Size.dwPhyBytes );
 
 		sprintf( tmp, "coarse-lamp-swap%u.raw", i );
 
-		dumpPicInit( &m_ScanParam, tmp );
-		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwPhyBytes );
+		dumpPicInit(&m_ScanParam, tmp);
+		dumpPic(tmp, (u_char*)scanbuf, m_ScanParam.Size.dwPhyBytes, 0);
 
 		dwDiv   = 10;
 		dwLoop1 = m_ScanParam.Size.dwPhyPixels/dwDiv;
@@ -341,7 +360,8 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 			for (dwLoop2 = dwDiv, dwR = dwG = dwB = 0; dwLoop2; dwLoop2--, dw++) {
 
 				if( m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-					if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+
+					if( usb_IsCISDevice(dev)) {
 						dwR += ((u_short*)scanbuf)[dw];
 						dwG += ((u_short*)scanbuf)
 						       [dw+m_ScanParam.Size.dwPhyPixels+1];
@@ -410,7 +430,7 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 		if((res_r == 10) && (res_g == 10) && (res_b == 10))
 			break;
 
-		/* we raise the gain for channel, that have been limited */
+		/* we raise the gain for channels, that have been limited */
 #ifdef _TWEAK_GAIN
 		if( res_r == 10 ) {
 			if( dev->usbDev.a_bRegs[0x3b] < 0xf)
@@ -436,7 +456,7 @@ static int cano_AdjustLightsource( Plustek_Device *dev )
 			DBG(_DBG_INFO, "* 10 times limit reached, still too dark!!!\n");
 			break;
 		}
-		usb_AdjustLamps(dev);
+		usb_AdjustLamps(dev, SANE_TRUE);
 	}
 
 	DBG( _DBG_INFO, "* red_lamp_on    = %u\n", hw->red_lamp_on  );
@@ -486,12 +506,13 @@ cano_adjGainSetting( u_char *min, u_char *max, u_char *gain,u_long val )
  *
  * adjLightsource, above, steals most of this function's thunder.
  */
-static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
+static SANE_Bool
+cano_AdjustGain( Plustek_Device *dev )
 {
 	char      tmp[40];
 	int       i = 0, adj = 1;
 	u_long    dw;
-	u_char   *scanbuf = dev->scanning.pScanBuffer;
+	u_long   *scanbuf = dev->scanning.pScanBuffer;
 	DCapsDef *scaps   = &dev->usbDev.Caps;
 	HWDef    *hw      = &dev->usbDev.HwSetting;
 
@@ -506,13 +527,15 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 	min[0] = min[1] = min[2] = 1;
 
 	DBG( _DBG_INFO, "cano_AdjustGain()\n" );
-	if((dev->adj.rgain != -1) && 
-	   (dev->adj.ggain != -1) && (dev->adj.bgain != -1)) {
-		setAdjGain( dev->adj.rgain, &dev->usbDev.a_bRegs[0x3b] );
-		setAdjGain( dev->adj.ggain, &dev->usbDev.a_bRegs[0x3c] );
-		setAdjGain( dev->adj.bgain, &dev->usbDev.a_bRegs[0x3d] );
-		DBG( _DBG_INFO, "- function skipped, using frontend values!\n" );
-		return SANE_TRUE;
+	if( !usb_InCalibrationMode(dev)) {
+		if((dev->adj.rgain != -1) && 
+		   (dev->adj.ggain != -1) && (dev->adj.bgain != -1)) {
+			setAdjGain( dev->adj.rgain, &dev->usbDev.a_bRegs[0x3b] );
+			setAdjGain( dev->adj.ggain, &dev->usbDev.a_bRegs[0x3c] );
+			setAdjGain( dev->adj.bgain, &dev->usbDev.a_bRegs[0x3d] );
+			DBG( _DBG_INFO, "- function skipped, using frontend values!\n" );
+			return SANE_TRUE;
+		}
 	}
 
 	/* define the strip to scan for coarse calibration
@@ -524,10 +547,8 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 
 	m_ScanParam.Size.dwBytes  = m_ScanParam.Size.dwPixels * 2;
 
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-    	m_ScanParam.bDataType == SCANDATATYPE_Color ) {
+	if( usb_IsCISDevice(dev) && m_ScanParam.bDataType == SCANDATATYPE_Color)
 		m_ScanParam.Size.dwBytes *=3;
-	}
 
 	m_ScanParam.Origin.x = (u_short)((u_long) hw->wActivePixelsStart *
 	                                                300UL / scaps->OpticDpi.x);
@@ -558,8 +579,8 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 
 		sprintf( tmp, "coarse-gain-%u.raw", i++ );
 
-		dumpPicInit( &m_ScanParam, tmp );
-		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwPhyBytes );
+		dumpPicInit(&m_ScanParam, tmp);
+		dumpPic(tmp, (u_char*)scanbuf, m_ScanParam.Size.dwPhyBytes, 0);
 
 		if(usb_HostSwap())
 			usb_Swap((u_short *)scanbuf, m_ScanParam.Size.dwPhyBytes );
@@ -579,7 +600,7 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 				/* do some averaging... */
 				for (dwLoop2 = dwDiv, dwR=dwG=dwB=0; dwLoop2; dwLoop2--, dw++) {
 
-					if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+					if( usb_IsCISDevice(dev)) {
 						dwR += ((u_short*)scanbuf)[dw];
 						dwG += ((u_short*)scanbuf)
 						       [dw+m_ScanParam.Size.dwPhyPixels+1];
@@ -634,13 +655,72 @@ static SANE_Bool cano_AdjustGain( Plustek_Device *dev )
 	return SANE_TRUE;
 }
 
+static int tweak_offset[3];
+
 /**
  */
-static int cano_GetNewOffset( Plustek_Device *dev, u_long *val, int channel, 
-                              signed char *low, signed char *now, signed char *high )
+static int
+cano_GetNewOffset(Plustek_Device *dev, u_long *val, int channel, signed char *low,
+                  signed char *now, signed char *high, u_long *zc)
 {
-	/* if we're too black, we're likely off the low end */
-	if( val[channel] <= 16 ) {
+	DCapsDef *scaps = &dev->usbDev.Caps;
+
+	if (tweak_offset[channel]) {
+
+		/* if we're too black, we're likely off the low end */
+		if( val[channel] <= 16 ) {
+			low[channel] =  now[channel];
+			now[channel] = (now[channel]+high[channel])/2;
+
+			dev->usbDev.a_bRegs[0x38+channel]= (now[channel]&0x3f);
+
+			if( low[channel]+1 >= high[channel] )
+				return 0;
+			return 1;
+
+		} else if ( val[channel]>=2048 ) {
+			high[channel]=now[channel];
+			now[channel]=(now[channel]+low[channel])/2;
+
+			dev->usbDev.a_bRegs[0x38+channel]= (now[channel]&0x3f);
+
+			if(low[channel]+1>=high[channel])
+				return 0;
+			return 1;
+		}
+	}
+
+	if (!(scaps->workaroundFlag & _WAF_INC_DARKTGT)) {
+		DBG( _DBG_INFO, "0 Pixel adjustment not active!\n");
+		return 0;
+	}
+
+	/* reaching this point, our black level should be okay, but
+	 * we also should check the percentage of 0 level pixels.
+	 * It turned out, that when having a lot of 0 level pixels,
+	 * the calibration will be bad and the resulting scans show up
+	 * stripes...
+	 */
+	if (zc[channel] > _DARK_TGT_THRESH) {
+		DBG( _DBG_INFO2, "More than %u%% 0 pixels detected, raise offset!\n",
+		                 _DARK_TGT_THRESH);
+		high[channel]=now[channel];
+		now[channel]=(now[channel]+low[channel])/2;
+
+		/* no more value checks, the goal to set the black level < 2048
+		 * will cause stripes...
+		 */
+		tweak_offset[channel] = 0;
+
+		dev->usbDev.a_bRegs[0x38+channel]= (now[channel]&0x3f);
+
+		if( low[channel]+1 >= high[channel] )
+			return 0;
+		return 1;
+
+	}
+#if 0
+	else if ( val[channel]>=4096 ) {
 		low[channel] =  now[channel];
 		now[channel] = (now[channel]+high[channel])/2;
 
@@ -649,17 +729,8 @@ static int cano_GetNewOffset( Plustek_Device *dev, u_long *val, int channel,
 		if( low[channel]+1 >= high[channel] )
 			return 0;
 		return 1;
-
-	} else if ( val[channel]>=2048 ) {
-		high[channel]=now[channel];
-		now[channel]=(now[channel]+low[channel])/2;
-
-		dev->usbDev.a_bRegs[0x38+channel]= (now[channel]&0x3f);
-
-		if(low[channel]+1>=high[channel])
-			return 0;
-		return 1;
 	}
+#endif
 	return 0;
 }
 
@@ -677,19 +748,20 @@ static int cano_GetNewOffset( Plustek_Device *dev, u_long *val, int channel,
    Plustek's example code disagrees with NatSemi's docs; going by the
    docs works better, I will assume the docs are correct. --Monty */
 
-static int cano_AdjustOffset( Plustek_Device *dev )
+static int
+cano_AdjustOffset( Plustek_Device *dev )
 {
 	char    tmp[40];
 	int     i, adj;
 	u_short r, g, b;
 	u_long  dw, dwPixels;
-	u_long  dwSum[3];
+	u_long  dwSum[3], zCount[3];
 
 	signed char low[3]  = {-32,-32,-32 };
 	signed char now[3]  = {  0,  0,  0 };
 	signed char high[3] = { 31, 31, 31 };
 
-	u_char   *scanbuf = dev->scanning.pScanBuffer;
+	u_long   *scanbuf = dev->scanning.pScanBuffer;
 	HWDef    *hw      = &dev->usbDev.HwSetting;
 	DCapsDef *scaps   = &dev->usbDev.Caps;
 	
@@ -697,29 +769,29 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 		return SANE_FALSE;
 
 	DBG( _DBG_INFO, "cano_AdjustOffset()\n" );
-	if((dev->adj.rofs != -1) &&
-	   (dev->adj.gofs != -1) && (dev->adj.bofs != -1)) {
-		dev->usbDev.a_bRegs[0x38] = (dev->adj.rofs & 0x3f);
-		dev->usbDev.a_bRegs[0x39] = (dev->adj.gofs & 0x3f);
-		dev->usbDev.a_bRegs[0x3a] = (dev->adj.bofs & 0x3f);
-		DBG( _DBG_INFO, "- function skipped, using frontend values!\n" );
-		return SANE_TRUE;
+	if( !usb_InCalibrationMode(dev)) {
+		if((dev->adj.rofs != -1) &&
+		   (dev->adj.gofs != -1) && (dev->adj.bofs != -1)) {
+			dev->usbDev.a_bRegs[0x38] = (dev->adj.rofs & 0x3f);
+			dev->usbDev.a_bRegs[0x39] = (dev->adj.gofs & 0x3f);
+			dev->usbDev.a_bRegs[0x3a] = (dev->adj.bofs & 0x3f);
+			DBG( _DBG_INFO, "- function skipped, using frontend values!\n" );
+			return SANE_TRUE;
+		}
 	}
 
 	m_ScanParam.Size.dwLines  = 1;
 	m_ScanParam.Size.dwPixels = scaps->Normal.Size.x*scaps->OpticDpi.x/300UL;
 
-	if( hw->bReg_0x26 & _ONE_CH_COLOR )
+	if( usb_IsCISDevice(dev))
 		dwPixels = m_ScanParam.Size.dwPixels;
 	else
-		dwPixels = (u_long)(hw->bOpticBlackEnd - hw->bOpticBlackStart );
+		dwPixels = (u_long)(hw->bOpticBlackEnd - hw->bOpticBlackStart);
 
 	m_ScanParam.Size.dwBytes = m_ScanParam.Size.dwPixels * 2;
 
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-		m_ScanParam.bDataType == SCANDATATYPE_Color ) {
+	if( usb_IsCISDevice(dev) && m_ScanParam.bDataType == SCANDATATYPE_Color)
 		m_ScanParam.Size.dwBytes *= 3;
-	}
 
 	m_ScanParam.Origin.x = (u_short)((u_long)hw->bOpticBlackStart * 300UL /
 	                                              dev->usbDev.Caps.OpticDpi.x);
@@ -736,6 +808,10 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 	DBG( _DBG_INFO2, "dwPhyBytes  = %lu\n", m_ScanParam.Size.dwPhyBytes );
 	DBG( _DBG_INFO2, "dwPhyPixels = %lu\n", m_ScanParam.Size.dwPhyPixels );
 
+	tweak_offset[0] =
+	tweak_offset[1] =
+	tweak_offset[2] = 1;
+
 	for( i = 0, adj = 1; adj != 0; i++ ) {
 
 		if((!usb_ScanBegin(dev, SANE_FALSE)) ||
@@ -747,8 +823,8 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 
 		sprintf( tmp, "coarse-off-%u.raw", i );
 
-		dumpPicInit( &m_ScanParam, tmp );
-		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwPhyBytes );
+		dumpPicInit(&m_ScanParam, tmp);
+		dumpPic(tmp, (u_char*)scanbuf, m_ScanParam.Size.dwPhyBytes, 0);
 
 		if(usb_HostSwap())
 			usb_Swap((u_short *)scanbuf, m_ScanParam.Size.dwPhyBytes );
@@ -756,10 +832,11 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 		if( m_ScanParam.bDataType == SCANDATATYPE_Color ) {
 
 			dwSum[0] = dwSum[1] = dwSum[2] = 0;
+			zCount[0] = zCount[1] = zCount[2] = 0;
 
 			for (dw = 0; dw < dwPixels; dw++) {
 
-				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+				if( usb_IsCISDevice(dev)) {
 
 					r = ((u_short*)scanbuf)[dw];
 					g = ((u_short*)scanbuf)[dw+m_ScanParam.Size.dwPhyPixels+1];
@@ -775,23 +852,33 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 				dwSum[1] += g;
 				dwSum[2] += b;
 
+				if (r==0) zCount[0]++;
+				if (g==0) zCount[1]++;
+				if (b==0) zCount[2]++;
 			}
 
-			DBG( _DBG_INFO2, "RedSum   = %lu, ave = %lu\n",
-			                            dwSum[0], dwSum[0]/dwPixels );
-			DBG( _DBG_INFO2, "GreenSum = %lu, ave = %lu\n",
-			                           dwSum[1], dwSum[1] /dwPixels );
-			DBG( _DBG_INFO2, "BlueSum  = %lu, ave = %lu\n",
-			                           dwSum[2], dwSum[2] /dwPixels );
+			DBG( _DBG_INFO2, "RedSum   = %lu, ave = %lu, ZC=%lu, %lu%%\n",
+			                        dwSum[0], dwSum[0]/dwPixels,
+			                        zCount[0], (zCount[0]*100)/dwPixels);
+			DBG( _DBG_INFO2, "GreenSum = %lu, ave = %lu, ZC=%lu, %lu%%\n",
+			                        dwSum[1], dwSum[1]/dwPixels,
+			                        zCount[1], (zCount[1]*100)/dwPixels);
+			DBG( _DBG_INFO2, "BlueSum  = %lu, ave = %lu, ZC=%lu, %lu%%\n",
+			                        dwSum[2], dwSum[2]/dwPixels,
+			                        zCount[2], (zCount[2]*100)/dwPixels);
 
 			/* do averaging for each channel */
 			dwSum[0] /= dwPixels;
 			dwSum[1] /= dwPixels;
 			dwSum[2] /= dwPixels;
 
-			adj  = cano_GetNewOffset( dev, dwSum, 0, low, now, high );
-			adj |= cano_GetNewOffset( dev, dwSum, 1, low, now, high );
-			adj |= cano_GetNewOffset( dev, dwSum, 2, low, now, high );
+			zCount[0] = (zCount[0] * 100)/ dwPixels;
+			zCount[1] = (zCount[1] * 100)/ dwPixels;
+			zCount[2] = (zCount[2] * 100)/ dwPixels;
+
+			adj  = cano_GetNewOffset(dev, dwSum, 0, low, now, high, zCount);
+			adj |= cano_GetNewOffset(dev, dwSum, 1, low, now, high, zCount);
+			adj |= cano_GetNewOffset(dev, dwSum, 2, low, now, high, zCount);
 
 			DBG( _DBG_INFO2, "RedOff   = %d/%d/%d\n",
 			                            (int)low[0],(int)now[0],(int)high[0]);
@@ -802,14 +889,23 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 
 		} else {
 			dwSum[0] = 0;
+			zCount[0] = 0;
 
-			for( dw = 0; dw < dwPixels; dw++ )
+			for( dw = 0; dw < dwPixels; dw++ ) {
 				dwSum[0] += ((u_short*)scanbuf)[dw];
 
-			dwSum[0] /= dwPixels;
-			DBG( _DBG_INFO2, "Sum=%lu, ave=%lu\n", dwSum[0],dwSum[0]/dwPixels);
+				if (((u_short*)scanbuf)[dw] == 0)
+					zCount[0]++;
+			}
 
-			adj = cano_GetNewOffset( dev, dwSum, 0, low, now, high );
+			DBG( _DBG_INFO2, "Sum=%lu, ave=%lu, ZC=%lu, %lu%%\n",
+			                  dwSum[0],dwSum[0]/dwPixels,
+			                  zCount[0], (zCount[0]*100)/dwPixels);
+
+			dwSum[0] /= dwPixels;
+			zCount[0] = (zCount[0] * 100)/ dwPixels;
+
+			adj = cano_GetNewOffset(dev, dwSum, 0, low, now, high, zCount);
 
 			dev->usbDev.a_bRegs[0x3a] =
 			dev->usbDev.a_bRegs[0x39] = dev->usbDev.a_bRegs[0x38];
@@ -825,14 +921,15 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 		_UIO(sanei_lm983x_write(dev->fd, 0x38, &dev->usbDev.a_bRegs[0x38], 3, SANE_TRUE));
 	}
 
+	/* is that really needed?! */
 	if( m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-		dev->usbDev.a_bRegs[0x38] = now[0];
-		dev->usbDev.a_bRegs[0x39] = now[1];
-		dev->usbDev.a_bRegs[0x3a] = now[2];
+		dev->usbDev.a_bRegs[0x38] = now[0] & 0x3f;
+		dev->usbDev.a_bRegs[0x39] = now[1] & 0x3f;
+		dev->usbDev.a_bRegs[0x3a] = now[2] & 0x3f;
 	} else {
 		dev->usbDev.a_bRegs[0x38] =
 		dev->usbDev.a_bRegs[0x39] =
-		dev->usbDev.a_bRegs[0x3a] = now[0];
+		dev->usbDev.a_bRegs[0x3a] = now[0] & 0x3f;
 	}
 
 	DBG( _DBG_INFO, "cano_AdjustOffset() done.\n" );
@@ -841,16 +938,14 @@ static int cano_AdjustOffset( Plustek_Device *dev )
 
 /** usb_AdjustDarkShading
  * fine calibration part 1
- *
  */
-static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
+static SANE_Bool
+cano_AdjustDarkShading( Plustek_Device *dev, u_short cal_dpi )
 {
 	char         tmp[40];
 	ScanParam   *param   = &dev->scanning.sParam;
 	ScanDef     *scan    = &dev->scanning;
-	DCapsDef    *scaps   = &dev->usbDev.Caps;
-	HWDef       *hw      = &dev->usbDev.HwSetting;
-	u_char      *scanbuf = scan->pScanBuffer;
+	u_long      *scanbuf = scan->pScanBuffer;
 	u_short     *bufp;
 	unsigned int i, j;
 	int          step, stepW, val;
@@ -860,36 +955,17 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 	if( usb_IsEscPressed())
 		return SANE_FALSE;
 
-	m_ScanParam = scan->sParam;
-
-#if 0
-	if( m_ScanParam.PhyDpi.x > 75)
-		m_ScanParam.Size.dwLines = 64;
-	else
-#endif
-		m_ScanParam.Size.dwLines = 32;
-
-	m_ScanParam.Origin.y  = 0;
-	m_ScanParam.bBitDepth = 16;
-
-	m_ScanParam.UserDpi.y    = scaps->OpticDpi.y;
-	m_ScanParam.Size.dwBytes = m_ScanParam.Size.dwPixels * 2;
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-		m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-		m_ScanParam.Size.dwBytes *= 3;
-	}
-
+	usb_PrepareFineCal( dev, &m_ScanParam, cal_dpi );
 	m_ScanParam.bCalibration = PARAM_DarkShading;
-	m_ScanParam.dMCLK        = dMCLK;
 
 	sprintf( tmp, "fine-dark.raw" );
-	dumpPicInit( &m_ScanParam, tmp );
+	dumpPicInit(&m_ScanParam, tmp);
 
 	usb_SetScanParameters( dev, &m_ScanParam );
 	if( usb_ScanBegin( dev, SANE_FALSE ) &&
 	    usb_ScanReadImage( dev, scanbuf, m_ScanParam.Size.dwTotalBytes)) {
 
-		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwTotalBytes );
+		dumpPic(tmp, (u_char*)scanbuf, m_ScanParam.Size.dwTotalBytes, 0);
 
 		if(usb_HostSwap())
 			usb_Swap((u_short *)scanbuf, m_ScanParam.Size.dwTotalBytes);
@@ -903,7 +979,7 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 	if( scan->sParam.bDataType == SCANDATATYPE_Color ) {
 		
 		stepW = m_ScanParam.Size.dwPhyPixels;
-		if( hw->bReg_0x26 & _ONE_CH_COLOR ) 
+		if( usb_IsCISDevice(dev))
 			step = m_ScanParam.Size.dwPhyPixels + 1;
 		else
 			step = (m_ScanParam.Size.dwPhyPixels*3) + 1;
@@ -913,14 +989,14 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 			red   = 0;
 			green = 0;
 			blue  = 0;
-			if( hw->bReg_0x26 & _ONE_CH_COLOR )
+			if( usb_IsCISDevice(dev))
 				bufp = ((u_short *)scanbuf)+i;
 			else
 				bufp = ((u_short *)scanbuf)+(i*3);
 
 			for( j=0; j<m_ScanParam.Size.dwPhyLines; j++ ) {
 
-				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+				if( usb_IsCISDevice(dev)) {
 					red   += *bufp; bufp+=step;
 					green += *bufp; bufp+=step;
 					blue  += *bufp; bufp+=step;
@@ -970,7 +1046,7 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 			}
 			a_wDarkShading[i]= gray/j + param->swOffset[0];
 		}
-			
+
 		memcpy( a_wDarkShading + m_ScanParam.Size.dwPhyPixels,
 		        a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2);
 		memcpy( a_wDarkShading + m_ScanParam.Size.dwPhyPixels * 2,
@@ -978,7 +1054,7 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
 	}
 
 	if(usb_HostSwap())
-		usb_Swap(a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
+		usb_Swap(a_wDarkShading, m_ScanParam.Size.dwPhyPixels * 2 * 3);
 
 	usb_line_statistics( "Dark", a_wDarkShading, m_ScanParam.Size.dwPhyPixels,
 	                     scan->sParam.bDataType == SCANDATATYPE_Color?1:0);
@@ -991,14 +1067,13 @@ static SANE_Bool cano_AdjustDarkShading( Plustek_Device *dev )
  * fine calibration part 2 - read the white calibration area and calculate
  * the gain coefficient for each pixel
  */
-static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
+static SANE_Bool
+cano_AdjustWhiteShading( Plustek_Device *dev, u_short cal_dpi )
 {
 	char         tmp[40];
 	ScanParam   *param   = &dev->scanning.sParam;
 	ScanDef     *scan    = &dev->scanning;
-	DCapsDef    *scaps   = &dev->usbDev.Caps;
-	HWDef       *hw      = &dev->usbDev.HwSetting;
-	u_char      *scanbuf = scan->pScanBuffer;
+	u_long      *scanbuf = scan->pScanBuffer;
 	u_short     *bufp;
 	unsigned int i, j;
 	int          step, stepW;
@@ -1008,27 +1083,8 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 	if( usb_IsEscPressed())
 		return SANE_FALSE;
 
-	m_ScanParam = scan->sParam;
-#if 0
-	if( m_ScanParam.PhyDpi.x > 75)
-		m_ScanParam.Size.dwLines = 64;
-	else
-#endif
-		m_ScanParam.Size.dwLines = 32;
-
-	m_ScanParam.Origin.y  = 0;
-	m_ScanParam.bBitDepth = 16;
-
-	m_ScanParam.UserDpi.y    = scaps->OpticDpi.y;
-	m_ScanParam.Size.dwBytes = m_ScanParam.Size.dwPixels * 2;
-
-	if( hw->bReg_0x26 & _ONE_CH_COLOR &&
-		m_ScanParam.bDataType == SCANDATATYPE_Color ) {
-		m_ScanParam.Size.dwBytes *= 3;
-	}
-
+	usb_PrepareFineCal( dev, &m_ScanParam, cal_dpi );
 	m_ScanParam.bCalibration = PARAM_WhiteShading;
-	m_ScanParam.dMCLK        = dMCLK;
 
 	sprintf( tmp, "fine-white.raw" );
 	DBG( _DBG_INFO2, "FINE WHITE Calibration Strip: %s\n", tmp );
@@ -1036,13 +1092,13 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 	DBG( _DBG_INFO2, "Pixels      = %lu\n", m_ScanParam.Size.dwPixels );
 	DBG( _DBG_INFO2, "Bytes       = %lu\n", m_ScanParam.Size.dwBytes  );
 	DBG( _DBG_INFO2, "Origin.X    = %u\n",  m_ScanParam.Origin.x );
-	dumpPicInit( &m_ScanParam, tmp );
+	dumpPicInit(&m_ScanParam, tmp);
 
 	if( usb_SetScanParameters( dev, &m_ScanParam ) &&
 	    usb_ScanBegin( dev, SANE_FALSE ) &&
 	    usb_ScanReadImage( dev, scanbuf, m_ScanParam.Size.dwTotalBytes)) {
 
-		dumpPic( tmp, scanbuf, m_ScanParam.Size.dwTotalBytes );
+		dumpPic(tmp, (u_char*)scanbuf, m_ScanParam.Size.dwTotalBytes, 0);
 
 		if(usb_HostSwap())
 			usb_Swap((u_short *)scanbuf, m_ScanParam.Size.dwTotalBytes);
@@ -1060,7 +1116,7 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 	if( scan->sParam.bDataType == SCANDATATYPE_Color ) {
 
 		stepW = m_ScanParam.Size.dwPhyPixels;
-		if( hw->bReg_0x26 & _ONE_CH_COLOR ) 
+		if( usb_IsCISDevice(dev))
 			step = m_ScanParam.Size.dwPhyPixels + 1;
 		else
 			step = (m_ScanParam.Size.dwPhyPixels*3) + 1;
@@ -1070,14 +1126,14 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 			red   = 0;
 			green = 0;
 			blue  = 0;
-			if( hw->bReg_0x26 & _ONE_CH_COLOR )
+			if( usb_IsCISDevice(dev))
 				bufp = ((u_short *)scanbuf)+i;
 			else
 				bufp = ((u_short *)scanbuf)+(i*3);
 
 			for( j=0; j<m_ScanParam.Size.dwPhyLines; j++ ) {
 
-				if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+				if( usb_IsCISDevice(dev)) {
 					red   += *bufp; bufp+=step;
 					green += *bufp; bufp+=step;
 					blue  += *bufp; bufp+=step;
@@ -1099,8 +1155,6 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 			a_wWhiteShading[i+stepW*2] = (blue  > 65535 ? 65535:blue );
 		}
 
-		if(usb_HostSwap())
-			usb_Swap(a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
 	} else {
 
 		step = m_ScanParam.Size.dwPhyPixels + 1;
@@ -1117,8 +1171,6 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 
 			a_wWhiteShading[i]= (gray > 65535 ? 65535:gray);
 		}
-		if(usb_HostSwap())
-			usb_Swap(a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2 );
 
 		memcpy( a_wWhiteShading + m_ScanParam.Size.dwPhyPixels,
 		        a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2);
@@ -1126,6 +1178,11 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 		        a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2);
 	}
 
+	if(usb_HostSwap())
+		usb_Swap(a_wWhiteShading, m_ScanParam.Size.dwPhyPixels * 2 * 3 );
+
+	usb_SaveCalSetShading( dev, &m_ScanParam );
+
 	usb_line_statistics( "White", a_wWhiteShading, m_ScanParam.Size.dwPhyPixels,
 	                     scan->sParam.bDataType == SCANDATATYPE_Color?1:0);
 
@@ -1135,8 +1192,12 @@ static SANE_Bool cano_AdjustWhiteShading( Plustek_Device *dev )
 
 /** the entry function for the CIS calibration stuff.
  */
-static int cano_DoCalibration( Plustek_Device *dev )
+static int
+cano_DoCalibration( Plustek_Device *dev )
 {
+	u_short   dpi, idx, idx_end;
+	u_long    save_waf;
+	SANE_Bool skip_fine;
 	ScanDef  *scan  = &dev->scanning;
 	HWDef    *hw    = &dev->usbDev.HwSetting;
 	DCapsDef *scaps = &dev->usbDev.Caps;
@@ -1147,18 +1208,25 @@ static int cano_DoCalibration( Plustek_Device *dev )
 	DBG( _DBG_INFO, "cano_DoCalibration()\n" );
 
 	if( _IS_PLUSTEKMOTOR(hw->motorModel)){
-		DBG( _DBG_ERROR, "altCalibration can't work with this Plustek motor control setup\n" );
+		DBG( _DBG_ERROR, "altCalibration can't work with this "
+		                 "Plustek motor control setup\n" );
 		return SANE_FALSE; /* can't cal this  */
 	}
 
-	/* Don't allow calibration settings from the other driver to confuse our use of
-	 * a few of its functions.
+	/* Don't allow calibration settings from the other driver to confuse our
+	 * use of a few of its functions.
 	 */
-	scaps->workaroundFlag &= ~_WAF_SKIP_WHITEFINE; 
-	scaps->workaroundFlag &= ~_WAF_SKIP_FINE; 
-	scaps->workaroundFlag &= ~_WAF_BYPASS_CALIBRATION; 
+	save_waf = scaps->workaroundFlag;
+	scaps->workaroundFlag &= ~_WAF_SKIP_WHITEFINE;
+	scaps->workaroundFlag &= ~_WAF_SKIP_FINE;
+	scaps->workaroundFlag &= ~_WAF_BYPASS_CALIBRATION;
 
-	usb_SpeedTest( dev );
+	if( !dev->adj.cacheCalData && !usb_IsSheetFedDevice(dev))
+		usb_SpeedTest( dev );
+
+	/* here we handle that warmup stuff for CCD devices */
+	if( !usb_AutoWarmup( dev ))
+		return SANE_FALSE;
 
 	/* Set the shading position to undefined */
 	strip_state = 0;
@@ -1167,14 +1235,18 @@ static int cano_DoCalibration( Plustek_Device *dev )
 	usb_SetMCLK( dev, &scan->sParam );
 
 	if( !scan->skipCoarseCalib ) {
+
+		if( !usb_Wait4ScanSample( dev ))
+			return SANE_FALSE;
+
 		DBG( _DBG_INFO2, "###### ADJUST LAMP (COARSE)#######\n" );
-		if( cano_PrepareToReadWhiteCal(dev))
+		if( cano_PrepareToReadWhiteCal(dev, SANE_TRUE))
 			return SANE_FALSE;
 
 		dev->usbDev.a_bRegs[0x45] &= ~0x10;
 		if( !cano_AdjustLightsource(dev)) {
 			DBG( _DBG_ERROR, "Coarse Calibration failed!!!\n" );
-			return _E_INTERNAL;
+			return SANE_FALSE;
 		}
 
 		DBG( _DBG_INFO2, "###### ADJUST OFFSET (COARSE) ####\n" );
@@ -1183,46 +1255,107 @@ static int cano_DoCalibration( Plustek_Device *dev )
 		
 		if( !cano_AdjustOffset(dev)) {
 			DBG( _DBG_ERROR, "Coarse Calibration failed!!!\n" );
-			return _E_INTERNAL;
+			return SANE_FALSE;
 		}
 
 		DBG( _DBG_INFO2, "###### ADJUST GAIN (COARSE)#######\n" );
-		if(cano_PrepareToReadWhiteCal(dev))
+		if(cano_PrepareToReadWhiteCal(dev, SANE_FALSE))
 			return SANE_FALSE;
 		
 		if( !cano_AdjustGain(dev)) {
 			DBG( _DBG_ERROR, "Coarse Calibration failed!!!\n" );
-			return _E_INTERNAL;
+			return SANE_FALSE;
 		}
 	} else {
 		strip_state = 1;
 		DBG( _DBG_INFO2, "###### COARSE calibration skipped #######\n" );
 	}
 
-	DBG( _DBG_INFO2, "###### ADJUST DARK (FINE) ########\n" );
-	if(cano_PrepareToReadBlackCal(dev))
-		return SANE_FALSE;
+	skip_fine = SANE_FALSE;
+	idx_end   = 2;
+	if( dev->adj.cacheCalData || usb_IsSheetFedDevice(dev)) {
 
-	dev->usbDev.a_bRegs[0x45] |= 0x10;
-	if( !cano_AdjustDarkShading(dev)) {
-		DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
-		return _E_INTERNAL;
+		skip_fine = usb_FineShadingFromFile(dev);
+
+		/* we recalibrate in any case ! */
+		if( usb_InCalibrationMode(dev)) {
+			skip_fine = SANE_FALSE;
+			idx_end   = DIVIDER+1;
+
+			/* did I say any case? */
+			if (scan->sParam.bBitDepth != 8) {
+				skip_fine = SANE_TRUE;
+				DBG( _DBG_INFO2, "No fine calibration for non-8bit modes!\n" );
+			}
+
+		} else if( usb_IsSheetFedDevice(dev)) {
+
+			/* we only do the calibration upon request !*/
+			if( !skip_fine ) {
+				DBG( _DBG_INFO2, "SHEET-FED device, skip fine calibration!\n" );
+				skip_fine = SANE_TRUE;
+				scaps->workaroundFlag |= _WAF_BYPASS_CALIBRATION;
+			}
+		}
 	}
 
-	DBG( _DBG_INFO2, "###### ADJUST WHITE (FINE) #######\n" );
-	if(cano_PrepareToReadWhiteCal(dev))
-		return SANE_FALSE;
+	if( !skip_fine ) {
 
-	if(!usb_ModuleToHome( dev, SANE_TRUE ))
-		return SANE_FALSE;
+		for( idx = 1; idx < idx_end; idx++ ) {
 
-	if( !usb_ModuleMove(dev, MOVE_Forward,
-			(u_long)dev->usbDev.pSource->ShadingOriginY)) {
-		return _E_INTERNAL;
-	}
-	if( !cano_AdjustWhiteShading(dev)) {
-		DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
-		return _E_INTERNAL;
+			dpi = 0;
+			if( usb_InCalibrationMode(dev)) {
+				dpi = usb_get_res( scaps->OpticDpi.x, idx );
+
+				/* we might should check against device specific limit */
+				if(dpi < 50)
+					continue;
+			}
+
+			DBG( _DBG_INFO2, "###### ADJUST DARK (FINE) ########\n" );
+			if(cano_PrepareToReadBlackCal(dev))
+				return SANE_FALSE;
+
+			dev->usbDev.a_bRegs[0x45] |= 0x10;
+			if( !cano_AdjustDarkShading(dev, dpi)) {
+				DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
+				return SANE_FALSE;
+			}
+
+			DBG( _DBG_INFO2, "###### ADJUST WHITE (FINE) #######\n" );
+			if(cano_PrepareToReadWhiteCal(dev, SANE_FALSE))
+				return SANE_FALSE;
+
+			if( !usb_IsSheetFedDevice(dev)) {
+				if(!usb_ModuleToHome( dev, SANE_TRUE ))
+					return SANE_FALSE;
+
+				if( !usb_ModuleMove(dev, MOVE_Forward,
+					(u_long)dev->usbDev.pSource->ShadingOriginY)) {
+					return SANE_FALSE;
+				}
+			}
+			if( !cano_AdjustWhiteShading(dev, dpi)) {
+				DBG( _DBG_ERROR, "Fine Calibration failed!!!\n" );
+				return SANE_FALSE;
+			}
+
+			/* force to go back */
+			strip_state = 0;
+		}
+	} else {
+		DBG( _DBG_INFO2, "###### FINE calibration skipped #######\n" );
+
+		dev->usbDev.a_bRegs[0x45] |= 0x10;
+		strip_state = 2;
+
+		m_ScanParam = scan->sParam;
+		usb_GetPhyPixels( dev, &m_ScanParam );
+
+		usb_line_statistics( "Dark", a_wDarkShading, m_ScanParam.Size.dwPhyPixels,
+		                      m_ScanParam.bDataType == SCANDATATYPE_Color?1:0);
+		usb_line_statistics( "White", a_wWhiteShading, m_ScanParam.Size.dwPhyPixels,
+		                      m_ScanParam.bDataType == SCANDATATYPE_Color?1:0);
 	}
 
 	/* Lamp on if it's not */
@@ -1231,7 +1364,8 @@ static int cano_DoCalibration( Plustek_Device *dev )
 
 	/* home the sensor after calibration
 	 */
-	usb_ModuleToHome( dev, SANE_TRUE );
+	if( !usb_IsSheetFedDevice(dev))
+		usb_ModuleToHome( dev, SANE_TRUE );
 	scan->fCalibrated = SANE_TRUE;
 	
 	DBG( _DBG_INFO, "cano_DoCalibration() done\n" );
@@ -1241,11 +1375,13 @@ static int cano_DoCalibration( Plustek_Device *dev )
 	DBG( _DBG_INFO, "REG[0x3c] = %u\n", dev->usbDev.a_bRegs[0x3c] );
 	DBG( _DBG_INFO, "REG[0x3d] = %u\n", dev->usbDev.a_bRegs[0x3d] );
 	DBG( _DBG_INFO, "Static Offset:\n" );
-	DBG( _DBG_INFO, "REG[0x38] = %u\n", dev->usbDev.a_bRegs[0x38] );
-	DBG( _DBG_INFO, "REG[0x39] = %u\n", dev->usbDev.a_bRegs[0x39] );
-	DBG( _DBG_INFO, "REG[0x3a] = %u\n", dev->usbDev.a_bRegs[0x3a] );
+	DBG( _DBG_INFO, "REG[0x38] = %i\n", dev->usbDev.a_bRegs[0x38] );
+	DBG( _DBG_INFO, "REG[0x39] = %i\n", dev->usbDev.a_bRegs[0x39] );
+	DBG( _DBG_INFO, "REG[0x3a] = %i\n", dev->usbDev.a_bRegs[0x3a] );
 	DBG( _DBG_INFO, "-------------------------\n" );
 
+	scaps->workaroundFlag |= save_waf;
+
 	return SANE_TRUE;
 }
 
