@@ -7,7 +7,7 @@
  *  @brief Image processing functions for copying and scaling image lines.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2007 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -26,10 +26,13 @@
  * - 0.47 - added big-endian/little endian stuff
  * - 0.48 - fixed usb_ColorDuplicateGray16() and
  *          usb_ColorScaleGray16()
- *        - added usb_BWScaleFromColor() and usb_BWDuplicateFromColor
+ *        - added usb_BWScaleFromColor() and usb_BWDuplicateFromColor()
  *        - cleanup
  * - 0.49 - a_bRegs is now part of the device structure
  * - 0.50 - cleanup
+ * - 0.51 - added usb_ColorDuplicateGray16_2(), usb_ColorScaleGray16_2()
+ *          usb_BWScaleFromColor_2() and usb_BWDuplicateFromColor_2()
+ * - 0.52 - cleanup
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -80,26 +83,42 @@ static u_short  wSum, Mask;
 /*
  */
 static u_char BitTable[8] = {
-	0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 
+	0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01
 };
 
 static u_char BitsReverseTable[256] = {
-	0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, 0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
-	0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8, 0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
-	0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4, 0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
-	0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, 0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
-	0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2, 0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
-	0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea, 0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
-	0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6, 0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
-	0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee, 0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
-	0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1, 0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
-	0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9, 0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
-	0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5, 0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
-	0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed, 0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
-	0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3, 0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
-	0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb, 0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
-	0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7, 0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
-	0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef, 0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff
+	0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0,
+	0x10, 0x90, 0x50, 0xd0, 0x30, 0xb0, 0x70, 0xf0,
+	0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
+	0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8,
+	0x04, 0x84, 0x44, 0xc4, 0x24, 0xa4, 0x64, 0xe4,
+	0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
+	0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec,
+	0x1c, 0x9c, 0x5c, 0xdc, 0x3c, 0xbc, 0x7c, 0xfc,
+	0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
+	0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2,
+	0x0a, 0x8a, 0x4a, 0xca, 0x2a, 0xaa, 0x6a, 0xea,
+	0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
+	0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6,
+	0x16, 0x96, 0x56, 0xd6, 0x36, 0xb6, 0x76, 0xf6,
+	0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
+	0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe,
+	0x01, 0x81, 0x41, 0xc1, 0x21, 0xa1, 0x61, 0xe1,
+	0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
+	0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9,
+	0x19, 0x99, 0x59, 0xd9, 0x39, 0xb9, 0x79, 0xf9,
+	0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
+	0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5,
+	0x0d, 0x8d, 0x4d, 0xcd, 0x2d, 0xad, 0x6d, 0xed,
+	0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
+	0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3,
+	0x13, 0x93, 0x53, 0xd3, 0x33, 0xb3, 0x73, 0xf3,
+	0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
+	0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb,
+	0x07, 0x87, 0x47, 0xc7, 0x27, 0xa7, 0x67, 0xe7,
+	0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
+	0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef,
+	0x1f, 0x9f, 0x5f, 0xdf, 0x3f, 0xbf, 0x7f, 0xff
 };
 
 /************************ some helper functions ******************************/
@@ -579,7 +598,7 @@ static void usb_ColorDuplicateGray16( Plustek_Device *dev )
 	ScanDef  *scan = &dev->scanning;
 	SANE_Bool swap = usb_HostSwap();
 
-	usb_AverageColorByte( dev );
+	usb_AverageColorWord( dev );
 
 	if (scan->sParam.bSource == SOURCE_ADF) {
 		next   = -1;
@@ -628,6 +647,71 @@ static void usb_ColorDuplicateGray16( Plustek_Device *dev )
 	}
 }
 
+/**
+ */
+static void usb_ColorDuplicateGray16_2( Plustek_Device *dev )
+{
+	int       next;
+	u_char    ls;
+	u_long    dw, pixels;
+	HiLoDef   tmp;
+	ScanDef  *scan = &dev->scanning;
+	SANE_Bool swap = usb_HostSwap();
+
+	usb_AverageColorWord( dev );
+
+	if (scan->sParam.bSource == SOURCE_ADF) {
+		next   = -1;
+		pixels = scan->sParam.Size.dwPixels - 1;
+	} else {
+		next   = 1;
+		pixels = 0;
+	}
+	if( scan->dwFlag & SCANFLAG_RightAlign )
+		ls = Shift;
+	else
+		ls = 0;
+
+	switch(scan->fGrayFromColor) {
+	case 1:
+		if( swap ) {
+			for (dw = 0; dw < scan->sParam.Size.dwPixels; dw++, pixels += next) {
+				tmp = *((HiLoDef*)&scan->Red.pw[dw]);
+				scan->UserBuf.pw[pixels] = _HILO2WORD(tmp) >> ls;
+			}
+		} else {
+			for (dw = 0; dw < scan->sParam.Size.dwPixels; dw++, pixels += next) {
+				scan->UserBuf.pw[pixels] = scan->Red.pw[dw] >> ls;
+			}
+		}
+		break;
+	case 2:
+		if( swap ) {
+			for (dw = 0; dw < scan->sParam.Size.dwPixels; dw++, pixels += next) {
+				tmp = *((HiLoDef*)&scan->Green.pw[dw]);
+				scan->UserBuf.pw[pixels] = _HILO2WORD(tmp) >> ls;
+			}
+		} else {
+			for (dw = 0; dw < scan->sParam.Size.dwPixels; dw++, pixels += next) {
+				scan->UserBuf.pw[pixels] = scan->Green.pw[dw] >> ls;
+			}
+		}
+		break;
+	case 3:
+		if( swap ) {
+			for (dw = 0; dw < scan->sParam.Size.dwPixels; dw++, pixels += next) {
+				tmp = *((HiLoDef*)&scan->Blue.pw[dw]);
+				scan->UserBuf.pw[pixels] = _HILO2WORD(tmp) >> ls;
+			}
+		} else {
+			for (dw = 0; dw < scan->sParam.Size.dwPixels; dw++, pixels += next) {
+				scan->UserBuf.pw[pixels] = scan->Blue.pw[dw] >> ls;
+			}
+		}
+		break;
+	}
+}
+
 /**
  */
 static void usb_GrayDuplicate8( Plustek_Device *dev )
@@ -683,7 +767,7 @@ static void usb_GrayDuplicate16( Plustek_Device *dev )
 		if( swap )
 			*dest = (_PHILO2WORD(pwm)) >> ls;
 		else
-			*dest = *((u_short*)pwm) >> ls;
+			*dest = (_PLOHI2WORD(pwm)) >> ls;
 	}
 }
 
@@ -735,7 +819,7 @@ static void usb_BWDuplicate( Plustek_Device *dev )
 }
 
 /** generate binary data from one of the three color inputs according to the
- *  value in fGrayFromColor
+ *  value in fGrayFromColor (CCD version)
  */
 static void usb_BWDuplicateFromColor( Plustek_Device *dev )
 {
@@ -776,6 +860,46 @@ static void usb_BWDuplicateFromColor( Plustek_Device *dev )
 	}
 }
 
+/** generate binary data from one of the three color inputs according to the
+ *  value in fGrayFromColor (CIS version)
+ */
+static void usb_BWDuplicateFromColor_2( Plustek_Device *dev )
+{
+	int      next;
+	u_char   d, *dest, *src;
+	u_short  j;
+	u_long   pixels;
+	ScanDef *scan = &dev->scanning;
+
+	if( scan->sParam.bSource == SOURCE_ADF ) {
+		dest = scan->UserBuf.pb + scan->sParam.Size.dwPixels - 1;
+		next = -1;
+	} else {
+		dest = scan->UserBuf.pb;
+		next = 1;
+	}
+
+	switch(scan->fGrayFromColor) {
+		case 1:  src = scan->Red.pb;   break;
+		case 3:  src = scan->Blue.pb;  break;
+		default: src = scan->Green.pb; break;
+	}
+
+	d = j = 0;
+	for( pixels = scan->sParam.Size.dwPixels; pixels; pixels--, src++ ) {
+
+		if( *src != 0 )
+			d |= BitTable[j];
+		j++;
+		if( j == 8 ) {
+			*dest = d;
+			dest += next;
+
+			d = j = 0;
+		}
+	}
+}
+
 /************************** the scaling functions ****************************/
 
 /**
@@ -954,6 +1078,97 @@ static void usb_ColorScaleGray16( Plustek_Device *dev )
 	}
 }
 
+/**
+ */
+static void usb_ColorScaleGray16_2( Plustek_Device *dev )
+{
+	u_char    ls;
+	int       izoom, ddax, next;
+	u_long    dw, pixels, bitsput;
+	HiLoDef   tmp;
+	SANE_Bool swap = usb_HostSwap();
+	ScanDef  *scan = &dev->scanning;
+
+	usb_AverageColorByte( dev );
+
+	dw = scan->sParam.Size.dwPixels;
+
+	if( scan->sParam.bSource == SOURCE_ADF ) {
+		next   = -1;
+		pixels = scan->sParam.Size.dwPixels - 1;
+	} else {
+		next   = 1;
+		pixels = 0;
+	}
+
+	izoom = usb_GetScaler( scan );
+
+	if( scan->dwFlag & SCANFLAG_RightAlign )
+		ls = Shift;
+	else
+		ls = 0;
+
+	switch( scan->fGrayFromColor ) {
+
+	case 1:
+		for( bitsput = 0, ddax = 0; dw; bitsput++ ) {
+
+			ddax -= _SCALER;
+
+			while((ddax < 0) && (dw > 0)) {
+				if( swap ) {
+					tmp = *((HiLoDef*)&scan->Red.pw[bitsput]);
+					scan->UserBuf.pw[pixels] = _HILO2WORD(tmp) >> ls;
+				} else {
+					scan->UserBuf.pw[pixels] = scan->Red.pw[dw] >> ls;
+				}
+				pixels += next;
+				ddax   += izoom;
+				dw--;
+			}
+		}
+		break;
+
+	case 2:
+		for( bitsput = 0, ddax = 0; dw; bitsput++ ) {
+
+			ddax -= _SCALER;
+
+			while((ddax < 0) && (dw > 0)) {
+				if( swap ) {
+					tmp = *((HiLoDef*)&scan->Green.pw[bitsput]);
+					scan->UserBuf.pw[pixels] = _HILO2WORD(tmp) >> ls;
+				} else {
+					scan->UserBuf.pw[pixels] = scan->Green.pw[bitsput] >> ls;
+				}
+				pixels += next;
+				ddax   += izoom;
+				dw--;
+			}
+		}
+		break;
+
+	case 3:
+		for( bitsput = 0, ddax = 0; dw; bitsput++ ) {
+
+			ddax -= _SCALER;
+
+			while((ddax < 0) && (dw > 0)) {
+				if( swap ) {
+					tmp = *((HiLoDef*)&scan->Blue.pw[bitsput]);
+					scan->UserBuf.pw[pixels] = _HILO2WORD(tmp) >> ls;
+				} else {
+					scan->UserBuf.pw[pixels] = scan->Blue.pw[bitsput] >> ls;
+				}
+				pixels += next;
+				ddax   += izoom;
+				dw--;
+			}
+		}
+		break;
+	}
+}
+
 /** here we copy and scale from scanner world to user world...
  */
 static void usb_ColorScale8( Plustek_Device *dev )
@@ -1300,6 +1515,55 @@ static void usb_BWScaleFromColor( Plustek_Device *dev )
 	}
 }
 
+/**
+ */
+static void usb_BWScaleFromColor_2( Plustek_Device *dev )
+{
+	u_char        d, *dest, *src;
+	u_short       j;
+	u_long        pixels;
+	int           izoom, ddax, next;
+	ScanDef      *scan = &dev->scanning;
+
+	if (scan->sParam.bSource == SOURCE_ADF) {
+		dest = scan->UserBuf.pb + scan->sParam.Size.dwPixels - 1;
+		next = -1;
+	} else {
+		dest = scan->UserBuf.pb;
+		next = 1;
+	}
+
+	/* setup the source buffer */
+	switch(scan->fGrayFromColor) {
+	case 1:  src = scan->Red.pb;   break;
+	case 3:  src = scan->Blue.pb;  break;
+	default: src = scan->Green.pb; break;
+	}
+
+	izoom = usb_GetScaler( scan );
+	ddax  = 0;
+
+	d = j = 0;
+	for( pixels = scan->sParam.Size.dwPixels; pixels; src++ ) {
+	
+		ddax -= _SCALER;
+
+		while((ddax < 0) && (pixels > 0)) {
+
+			if( *src != 0 )
+				d |= BitTable[j];
+			j++;
+			if( j == 8 ) {
+				*dest = d;
+				dest += next;
+				d = j = 0;
+			}
+			ddax   += izoom;
+			pixels--;
+		}
+	}
+}
+
 /**
  */
 static void usb_GrayScale8( Plustek_Device *dev )
@@ -1379,7 +1643,7 @@ static void usb_GrayScale16( Plustek_Device *dev )
 			if( swap )
 				*dest = _PHILO2WORD(pwm) >> ls;
 			else
-				*dest = *((u_short*)pwm) >> ls;
+				*dest = _PLOHI2WORD(pwm) >> ls;
 
 			dest += next;
 			ddax += izoom;
@@ -1455,8 +1719,13 @@ static void usb_GetImageProc( Plustek_Device *dev )
 						DBG( _DBG_INFO, "ImageProc is: ColorScale16\n" );
 					}
 					if (scan->fGrayFromColor) {
-						scan->pfnProcess = usb_ColorScaleGray16;
-						DBG( _DBG_INFO, "ImageProc is: ColorScaleGray16\n" );
+						if( usb_IsCISDevice(dev)){ 
+							scan->pfnProcess = usb_ColorScaleGray16_2;
+							DBG( _DBG_INFO, "ImageProc is: ColorScaleGray16_2\n" );
+						} else {
+							scan->pfnProcess = usb_ColorScaleGray16;
+							DBG( _DBG_INFO, "ImageProc is: ColorScaleGray16\n" );
+						}
 					}
 				} else if (scan->dwFlag & SCANFLAG_Pseudo48) {
 					scan->pfnProcess = usb_ColorScalePseudo16;
@@ -1464,9 +1733,14 @@ static void usb_GetImageProc( Plustek_Device *dev )
 					
 				} else if (scan->fGrayFromColor) {
 
-					if( usb_IsCISDevice(dev)){ 
-						scan->pfnProcess = usb_ColorScaleGray_2;
-						DBG( _DBG_INFO, "ImageProc is: ColorScaleGray_2\n" );
+					if( usb_IsCISDevice(dev)){
+						if (scan->fGrayFromColor > 7 ) {
+							scan->pfnProcess = usb_BWScaleFromColor_2;
+							DBG( _DBG_INFO, "ImageProc is: BWScaleFromColor_2\n" );
+						} else {
+							scan->pfnProcess = usb_ColorScaleGray_2;
+							DBG( _DBG_INFO, "ImageProc is: ColorScaleGray_2\n" );
+						}
 					} else {
 						if (scan->fGrayFromColor > 7 ) {
 							scan->pfnProcess = usb_BWScaleFromColor;
@@ -1517,7 +1791,7 @@ static void usb_GetImageProc( Plustek_Device *dev )
 
 			case SCANDATATYPE_Color:
 				if (scan->sParam.bBitDepth > 8) {
-					if( usb_IsCISDevice(dev)){ 
+					if( usb_IsCISDevice(dev)){
 						scan->pfnProcess = usb_ColorDuplicate16_2;
 						DBG( _DBG_INFO, "ImageProc is: ColorDuplicate16_2\n" );
 					} else {
@@ -1525,16 +1799,26 @@ static void usb_GetImageProc( Plustek_Device *dev )
 						DBG( _DBG_INFO, "ImageProc is: ColorDuplicate16\n" );
 					}
 					if (scan->fGrayFromColor) {
-						scan->pfnProcess = usb_ColorDuplicateGray16;
-						DBG( _DBG_INFO, "ImageProc is: ColorDuplicateGray16\n" );
+						if( usb_IsCISDevice(dev)){
+							scan->pfnProcess = usb_ColorDuplicateGray16_2;
+							DBG( _DBG_INFO, "ImageProc is: ColorDuplicateGray16_2\n" );
+						} else {
+							scan->pfnProcess = usb_ColorDuplicateGray16;
+							DBG( _DBG_INFO, "ImageProc is: ColorDuplicateGray16\n" );
+						}
 					}
 				} else if (scan->dwFlag & SCANFLAG_Pseudo48) {
 					scan->pfnProcess = usb_ColorDuplicatePseudo16;
 					DBG( _DBG_INFO, "ImageProc is: ColorDuplicatePseudo16\n" );
 				} else if (scan->fGrayFromColor) {
-					if( usb_IsCISDevice(dev)){ 
-						scan->pfnProcess = usb_ColorDuplicateGray_2;
-						DBG( _DBG_INFO, "ImageProc is: ColorDuplicateGray_2\n" );
+					if( usb_IsCISDevice(dev)){
+						if (scan->fGrayFromColor > 7 ) {
+							scan->pfnProcess = usb_BWDuplicateFromColor_2;
+							DBG( _DBG_INFO, "ImageProc is: BWDuplicateFromColor_2\n" );
+						} else {
+							scan->pfnProcess = usb_ColorDuplicateGray_2;
+							DBG( _DBG_INFO, "ImageProc is: ColorDuplicateGray_2\n" );
+						}
 					} else {
 						if (scan->fGrayFromColor > 7 ) {
 							scan->pfnProcess = usb_BWDuplicateFromColor;
@@ -1665,7 +1949,7 @@ static SANE_Int usb_ReadData( Plustek_Device *dev )
 
 		if( usb_ScanReadImage( dev, scan->pbGetDataBuf, dw )) {
 
-			dumpPic( "plustek-pic.raw", scan->pbGetDataBuf, dw );
+			dumpPic("plustek-pic.raw", scan->pbGetDataBuf, dw, 0);
 
 			if( scan->dwLinesDiscard ) {
 
