@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 1996, 1997 David Mosberger-Tang
+   Copyright (C) Marian Eichholz 2001
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -37,10 +37,7 @@
    If you write modifications of your own for SANE, it is your choice
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.
-
-   This file implements a dynamic linking based SANE meta backend.  It
-   allows managing an arbitrary number of SANE backends by using
-   dynamic linking to load backends on demand.  */
+*/
 
 /* ======================================================================
 
@@ -274,23 +271,8 @@ TState StartScanColor(TInstance *this)
   UploadGammaTable(this,0x4000,this->agammaB);
   INST_ASSERT();
 
-#ifndef SM3600_NO_GAIN_CORRECTION
-  RegWrite(this,0x3D,1,0x0F | 0x80); /* 10XXXXXX : one offset table */
-  RegWrite(this,0x3F,1,0x18); /* 16KB gain at 0x06000 */
-  {
-    unsigned short uwGain[8192];
-    int i,iOff;
-
-    /*
-      Oopsi: correction data starts at the left of the scanning window!
-    */
-    iOff=this->param.x/2+this->calibration.xMargin;
-    for (i=iOff; i<MAX_PIXEL_PER_SCANLINE; i++)
-      uwGain[i-iOff]=this->calibration.achStripeY[i]<<4;
-    for (i=0; i<0x4000; i+=0x1000)
-      MemWriteArray(this,(0x6000+i)>>1,0x1000,(unsigned char*)&uwGain[i>>1]);
-  }
-#endif
+  UploadGainCorrection(this,0x6000);
+  INST_ASSERT();
 
   /* enough for 1/100 inch sensor distance */
   this->state.cBacklog=1+2*this->state.ySensorSkew;
