@@ -282,8 +282,12 @@ auth_callback (SANE_String_Const resource,
 
   if (query_user == 1)
     {
+#ifdef HAVE_GETPASS
       strcpy (password, (wipe = getpass ("Enter password: ")));
       memset (wipe, 0, strlen (password));
+#else
+      printf("OS has no getpass().  User Queries will not work\n");
+#endif
     }
 
   if (md5mode)
@@ -1192,8 +1196,6 @@ scan_it (void)
   SANE_Word total_bytes = 0, expected_bytes;
   SANE_Int hang_over = -1;
 
-  buffer = malloc (buffer_size);
-
   do
     {
       if (!first_frame)
@@ -2213,9 +2215,13 @@ List of available devices:", prog_name);
   if (output_format != OUTPUT_PNM)
     resolution_value = get_resolution ();
 
+#ifdef SIGHUP
   signal (SIGHUP, sighandler);
-  signal (SIGINT, sighandler);
+#endif
+#ifdef SIGPIPE
   signal (SIGPIPE, sighandler);
+#endif
+  signal (SIGINT, sighandler);
   signal (SIGTERM, sighandler);
 
   if (test == 0)
@@ -2240,6 +2246,8 @@ List of available devices:", prog_name);
         exit (1);
       }
 
+      buffer = malloc (buffer_size);
+
       do
 	{
 	  char path[PATH_MAX];
