@@ -40,15 +40,17 @@
 */
 
 /** @file sanei_config.h
- * This file provides generic configuration support.  
+ * Generic configuration support.
  *
  * Use the functions of this header file if you want to read and analyze
  * configuration files.
 */
 
 #ifndef sanei_config_h
+#define sanei_config_h 1
 
 #include <stdio.h>
+#include <sane/sane.h>
 
 /** Search configuration file \a name along directory list and return file
  * pointer if such a file exists.  
@@ -77,7 +79,7 @@ extern FILE *sanei_config_open (const char *name);
 */
 extern char *sanei_config_read (char *str, int n, FILE *stream);
 
-/** Removes all whitespace from the beginning of a string.
+/** Remove all whitespace from the beginning of a string.
  *
  * @param str string
  *
@@ -87,7 +89,7 @@ extern char *sanei_config_read (char *str, int n, FILE *stream);
 extern const char *sanei_config_skip_whitespace (const char *str);
 
 
-/** Scans a string constant from a line of text and returns a malloced copy
+/** Scan a string constant from a line of text and return a malloced copy
  * of it.
  *
  * It's the responsibility of the caller to free the returned string constant
@@ -122,6 +124,52 @@ extern const char *sanei_config_get_string (const char *str,
  */
 extern void sanei_config_attach_matching_devices (const char *name,
 						  SANE_Status (*attach)
-						    (const char *dev));
+						  (const char *dev));
+
+/** this structure holds the description of configuration options. There is
+ * a list for options and another for their values. 
+ * These lists are used when the configuration file is
+ * parsed. Read values are stored in Option_Value. Helpers functions are 
+ * provided to access values easily */
+typedef struct
+{
+  /** number of options */
+  SANE_Int count;
+
+  /** NULL terminated list of configuration option */
+  SANE_Option_Descriptor **descriptors;
+
+  /** values for the configuration options */
+  void **values;
+
+} SANEI_Config;
+
+/** Parse configuration file, reading configuration options and trying to
+ * attach devices found in file.
+ *
+ * The option are gathered in a single configuration structure. Each time
+ * a line holds a value that is not an option, the attach function is called
+ * with the name found and the configuration structure with it's current values.
+ *
+ * @param config_file name of the configuration file to read
+ * @param config configuration structure to be filled during configuration
+ *  	  parsing and passed to the attach callback function
+ * @param config_attach attach with config callback function
+ *
+ * @return SANE_STATUS_GOOD if no errors
+ *         SANE_STATUS_ACCESS_DENIED if configuration file can't be opened
+ */
+extern SANE_Status sanei_configure_attach (
+  const char *config_file,
+  SANEI_Config *config,
+  SANE_Status (*config_attach)(SANEI_Config *config, const char *devname)
+);
+
+/** Return the list of config directories, extracted from the SANE_CONFIG_DIR
+ * environment variable and the default paths.
+ * @return a string containing the configuration paths, separated by the
+ *         operating system's path separator
+ */
+extern const char *sanei_config_get_paths (void);
 
 #endif	/* sanei_config_h */
