@@ -48,7 +48,9 @@
 #endif
 #include <string.h>
 #include <stdarg.h>
+#ifdef HAVE_VSYSLOG
 #include <syslog.h>
+#endif
 #ifdef HAVE_OS2_H
 #include <sys/types.h>
 #endif
@@ -100,6 +102,7 @@ sanei_debug_msg
 	
   if (max_level >= level)
     {
+#ifdef S_IFSOCK
       if ( 1 == isfdtype(fileno(stderr), S_IFSOCK) )
 	{
 	  msg = (char *)malloc (sizeof(char) * (strlen(be) + strlen(fmt) + 4));
@@ -116,6 +119,7 @@ sanei_debug_msg
 	    }
 	}
       else
+#endif
 	{
 	  fprintf (stderr, "[%s] ", be);
           vfprintf (stderr, fmt, ap);
