@@ -64,7 +64,11 @@
 # define INCL_DOSPROCESS
 # include <os2.h>
 #endif
-#if !defined USE_PTHREAD && !defined HAVE_OS2_H
+#ifdef __BEOS__
+# undef USE_PTHREAD /* force */
+# include <kernel/OS.h>
+#endif
+#if !defined USE_PTHREAD && !defined HAVE_OS2_H && !defined __BEOS__
 # include <sys/wait.h>
 #endif
 #if defined USE_PTHREAD
@@ -105,17 +109,66 @@ sanei_thread_init( void )
 SANE_Bool
 sanei_thread_is_forked( void )
 {
-#if defined USE_PTHREAD || defined HAVE_OS2_H
+#if defined USE_PTHREAD || defined HAVE_OS2_H || defined __BEOS__
 	return SANE_FALSE;
 #else
 	return SANE_TRUE;
 #endif
 }
 
-int
-sanei_thread_kill( int pid )
+/* Use this to mark a SANE_Pid as invaild instead of marking with -1.
+ */
+static void
+sanei_thread_set_invalid( SANE_Pid *pid )
 {
-	DBG(2, "sanei_thread_kill() will kill %d\n", (int)pid);
+
+#ifdef WIN32
+	pid->p = 0;
+#else
+	*pid = -1;
+#endif
+}
+
+/* Return if PID is a valid PID or not. */
+SANE_Bool
+sanei_thread_is_invalid( SANE_Pid pid )
+{
+	SANE_Bool rc = SANE_FALSE;
+
+#ifdef WIN32
+	if (pid.p == 0)
+	    rc = SANE_TRUE;
+#else
+	if (pid == -1)
+	    rc = SANE_TRUE;
+#endif
+
+	return rc;
+}
+
+/* pthread_t is not an integer on all platform.  Do our best to return
+ * a PID-like value from structure.  On platforms were it is an integer,
+ * return that.
+ */
+static long
+sanei_thread_pid_to_long( SANE_Pid pid )
+{
+	int rc;
+
+#ifdef WIN32
+	rc = pid.p;
+#else
+	rc = pid;
+#endif
+
+	return rc;
+}
+
+int
+sanei_thread_kill( SANE_Pid pid )
+{
+	DBG(2, "sanei_thread_kill() will kill %ld\n",
+	    sanei_thread_pid_to_long(pid));
 #ifdef USE_PTHREAD
 #if defined (__APPLE__) && defined (__MACH__)
 	return pthread_kill((pthread_t)pid, SIGUSR2);
@@ -150,10 +203,10 @@ local_thread( void *arg )
  * args  pointer to scanner data structure
  *
  */
-int
+SANE_Pid
 sanei_thread_begin( int (*func)(void *args), void* args )
 {
-	int           pid;
+	SANE_Pid pid;
 
 	td.func      = func;
 	td.func_data = args;
@@ -168,8 +221,8 @@ sanei_thread_begin( int (*func)(void *args), void* args )
 	return pid;
 }
 
-int
-sanei_thread_waitpid( int pid, int *status )
+SANE_Pid
+sanei_thread_waitpid( SANE_Pid pid, int *status )
 {
   if (status)
     *status = 0;
@@ -177,12 +230,74 @@ sanei_thread_waitpid( int pid, int *status )
 }
 
 int
-sanei_thread_sendsig( int pid, int sig )
+sanei_thread_sendsig( SANE_Pid pid, int sig )
 {
 	return 0;
 }
 
-#else /* HAVE_OS2_H */
+#elif defined __BEOS__
+
+static int32
+local_thread( void *arg )
+{
+	pThreadDataDef ltd = (pThreadDataDef)arg;
+
+	DBG( 2, "thread started, calling func() now...\n" );
+	ltd->status = ltd->func( ltd->func_data );
+
+	DBG( 2, "func() done - status = %d\n", ltd->status );
+	return ltd->status;
+}
+
+/*
+ * starts a new thread or process
+ * parameters:
+ * star  address of reader function
+ * args  pointer to scanner data structure
+ *
+ */
+SANE_Pid
+sanei_thread_begin( int (*func)(void *args), void* args )
+{
+	SANE_Pid pid;
+
+	td.func      = func;
+	td.func_data = args;
+
+	pid = spawn_thread( local_thread, "sane thread (yes they can be)", B_NORMAL_PRIORITY, (void*)&td );
+	if ( pid < B_OK ) {
+		DBG( 1, "spawn_thread() failed\n" );
+		return -1;
+	}
+	if ( resume_thread(pid) < B_OK ) {
+		DBG( 1, "resume_thread() failed\n" );
+		return -1;
+	}
+   
+	DBG( 2, "spawn_thread() created thread %d\n", pid );
+	return pid;
+}
+
+SANE_Pid
+sanei_thread_waitpid( SANE_Pid pid, int *status )
+{
+  int32 st;
+  if ( wait_for_thread(pid, &st) < B_OK )
+    return -1;
+  if ( status )
+    *status = (int)st;
+  return pid;
+}
+
+int
+sanei_thread_sendsig( SANE_Pid pid, int sig )
+{
+	if (sig == SIGKILL)
+		sig = SIGKILLTHR;
+	return kill(pid, sig);
+}
+
+#else /* HAVE_OS2_H, __BEOS__ */
 
 #ifdef USE_PTHREAD
 
@@ -241,6 +356,7 @@ local_thread( void *arg )
 static void
 restore_sigpipe( void )
 {
+#ifdef SIGPIPE
 	struct sigaction act;
 
 	if( sigaction( SIGPIPE, NULL, &act ) == 0 ) {
@@ -254,12 +370,13 @@ restore_sigpipe( void )
 			sigaction( SIGPIPE, &act, NULL );
 		}
 	}
+#endif
 }
 
 #else /* the process stuff */
 
 static int
-eval_wp_result( int pid, int wpres, int pf )
+eval_wp_result( SANE_Pid pid, int wpres, int pf )
 {
 	int retval = SANE_STATUS_IO_ERROR;
 
@@ -282,13 +399,14 @@ eval_wp_result( int pid, int wpres, int pf )
 }
 #endif
 
-int
+SANE_Pid
 sanei_thread_begin( int (func)(void *args), void* args )
 {
-	int       pid;
 #ifdef USE_PTHREAD
-	struct sigaction act;
+	int result;
 	pthread_t thread;
+#ifdef SIGPIPE
+	struct sigaction act;
 
 	/* if signal handler for SIGPIPE is SIG_DFL, replace by SIG_IGN */
 	if( sigaction( SIGPIPE, NULL, &act ) == 0 ) {
@@ -302,21 +420,25 @@ sanei_thread_begin( int (func)(void *args), void* args )
 			sigaction( SIGPIPE, &act, NULL );
 		}
 	}
+#endif
 
 	td.func      = func;
 	td.func_data = args;
 
-	pid = pthread_create( &thread, NULL, local_thread, &td );
+	result = pthread_create( &thread, NULL, local_thread, &td );
 	usleep( 1 );
 
-	if ( pid != 0 ) {
-		DBG( 1, "pthread_create() failed with %d\n", pid );
-		return -1;
+	if ( result != 0 ) {
+		DBG( 1, "pthread_create() failed with %d\n", result );
+		sanei_thread_set_invalid(&thread);
 	}
+	else
+		DBG( 2, "pthread_create() created thread %ld\n",
+		     (SANE_Pid)thread );
 
-	DBG( 2, "pthread_create() created thread %d\n", (int)thread );
-	return (int)thread;
+	return (SANE_Pid)thread;
 #else
+	SANE_Pid pid;
 	pid = fork();
 	if( pid < 0 ) {
 		DBG( 1, "fork() failed\n" );
@@ -326,7 +448,7 @@ sanei_thread_begin( int (func)(void *args), void* args )
 	if( pid == 0 ) {
 
     	/* run in child context... */
-	    int status = func( args );
+		int status = func( args );
 		
 		/* don't use exit() since that would run the atexit() handlers */
 		_exit( status );
@@ -338,34 +460,37 @@ sanei_thread_begin( int (func)(void *args), void* args )
 }
 
 int
-sanei_thread_sendsig( int pid, int sig )
+sanei_thread_sendsig( SANE_Pid pid, int sig )
 {
+	DBG(2, "sanei_thread_sendsig() %d to thread (id=%ld)\n", sig,
+	    sanei_thread_pid_to_long(pid));
 #ifdef USE_PTHREAD
-	DBG(2, "sanei_thread_sendsig() %d to thread(id=%d)\n", sig, pid);
-	return pthread_kill((pthread_t)pid, sig );
+	return pthread_kill( (pthread_t)pid, sig );
 #else
-	DBG(2, "sanei_thread_sendsig() %d to process (id=%d)\n", sig, pid);
 	return kill( pid, sig );
 #endif
 }
 
-int
-sanei_thread_waitpid( int pid, int *status )
+SANE_Pid
+sanei_thread_waitpid( SANE_Pid pid, int *status )
 {
 #ifdef USE_PTHREAD
 	int *ls;
 #else
 	int ls;
 #endif
-	int  result, stat;
+	SANE_Pid result;
+	int stat;
 
 	stat = 0;
 
-	DBG(2, "sanei_thread_waitpid() - %d\n", pid);
+	DBG(2, "sanei_thread_waitpid() - %ld\n",
+	    sanei_thread_pid_to_long(pid));
 #ifdef USE_PTHREAD
-	result = pthread_join((pthread_t)pid, (void*)&ls );
+	int rc;
+	rc = pthread_join( (pthread_t)pid, (void*)&ls );
 
-	if( 0 == result ) {
+	if( 0 == rc ) {
 		if( PTHREAD_CANCELED == ls ) {
 			DBG(2, "* thread has been canceled!\n" );
 			stat = SANE_STATUS_GOOD;
@@ -378,7 +503,7 @@ sanei_thread_waitpid( int pid, int *status )
 	/* call detach in any case to make sure that the thread resources 
 	 * will be freed, when the thread has terminated
 	 */
-	DBG(2, "* detaching thread(%d)\n", pid );
+	DBG(2, "* detaching thread(%ld)\n", pid );
 	pthread_detach((pthread_t)pid);
 	if (status)
 		*status = stat;
@@ -402,9 +527,9 @@ sanei_thread_waitpid( int pid, int *status )
 #endif /* HAVE_OS2_H */
 
 SANE_Status
-sanei_thread_get_status( int pid )
+sanei_thread_get_status( SANE_Pid pid )
 {
-#if defined USE_PTHREAD || defined HAVE_OS2_H
+#if defined USE_PTHREAD || defined HAVE_OS2_H || defined __BEOS__
 	_VAR_NOT_USED( pid );
 
 	return td.status;
