@@ -30,10 +30,15 @@
 #include <dirent.h>
 #include <errno.h>
 
-#if defined (HAVE_WINDOWS_H)
-#include <windows.h>
-#include <ddk/scsi.h>
-#include <ddk/ntddscsi.h>
+#if defined (HAVE_DDK_NTDDSCSI_H) || defined (HAVE_NTDDSCSI_H)
+# define WIN32_SCSI
+# include <windows.h>
+# if defined (HAVE_DDK_NTDDSCSI_H)
+#  include <ddk/scsi.h>
+#  include <ddk/ntddscsi.h>
+# elif defined (HAVE_NTDDSCSI_H)
+#  include <ntddscsi.h>
+# endif
 #endif
 
 #include "../include/sane/sanei.h"
@@ -42,7 +47,11 @@
 #include "../include/sane/sanei_config.h"
 
 #ifdef HAVE_LIBUSB
+#ifdef HAVE_LUSB0_USB_H
+#include <lusb0_usb.h>
+#else
 #include <usb.h>
+#endif
 extern char * check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file);
 #endif
 
@@ -1144,7 +1153,7 @@ get_next_file (char *dir_name, DIR * dir)
   return file_name;
 }
 
-#if defined (HAVE_WINDOWS_H)
+#if defined(WIN32_SCSI)
 /* Return a list of potential scanners. There's a lot of hardcoded values here that might break on a system with lots of scsi devices. */
 static char **build_scsi_dev_list(void)
 {
@@ -1836,7 +1845,7 @@ main (int argc, char **argv)
 	0
       };
 
-#if defined (HAVE_WINDOWS_H) || \
+#if defined (WIN32_SCSI) || \
     defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
     defined (HAVE_IOKIT_SCSI_SCSICOMMANDOPERATIONCODES_H) || \
     defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
@@ -2072,12 +2081,14 @@ main (int argc, char **argv)
   if (verbose > 0)
     printf ("\n  # Most Scanners connected to the parallel port or other "
 	    "proprietary ports\n  # can't be detected by this program.\n");
+#ifdef HAVE_GETUID
   if (getuid ())
     if (verbose > 0)
       printf
 	("\n  # You may want to run this program as root to find all devices. "
 	 "Once you\n  # found the scanner devices, be sure to adjust access "
 	 "permissions as\n  # necessary.\n");
+#endif
 
   if (verbose > 1)
     printf ("done\n");
